from manga_py.provider import Provider
from .helpers.std import Std


class MangaPandaCom(Provider, Std):

    def get_chapter_index(self) -> str:
        idx = self.re.search(r'\.\w{2,7}/[^/]+/([^/]+)', self.chapter)
        return idx.group(1)

    def get_content(self):
        return self._get_content('{}/{}')

    def get_manga_name(self) -> str:
        return self._get_name(r'\.\w{2,7}/([^/]+)')

    def get_chapters(self):
        return self._elements('#listing a')

    def get_files(self):
        img_selector = '#imgholder img'
        url = self.http().normalize_uri(self.chapter)

        parser = self.html_fromstring(url, '#container', 0)
        count_pages = self._first_select_options(parser, '#selectpage')
        images = self._images_helper(parser, img_selector)

        n = 1
        while n < len(count_pages):
            parser = self.html_fromstring('{}/{}'.format(url, 1 + n))
            images += self._images_helper(parser, img_selector)
            n += 1

        return images

    def get_cover(self):
        return self._cover_from_content('#mangaimg img')

    def book_meta(self) -> dict:
        # todo meta
        pass


main = MangaPandaCom



from manga_py.provider import Provider
from .helpers.std import Std


class ComicOrg(Provider, Std):
    def get_chapter_index(self) -> str:
        return str(self.chapter_id)

    def get_content(self):
        return self.http_get(self.get_url())

    def get_manga_name(self) -> str:
        return 'Album_18comic {}'.format(self._get_name(r'/album/(\d+)'))

    def get_chapters(self):
        elements = self._elements('.episode > ul a[href^="/photo/"]')
        n = self.http().normalize_uri
        return list(set(map(lambda x: n(x.get('href')), elements)))[::-1]

    def get_files(self):
        parser = self.html_fromstring(self.chapter)
        return self._images_helper(parser, '.panel-body img[id]', 'data-original', 'src')

    def get_cover(self) -> str:
        return self._cover_from_content('.thumb-overlay img')


main = ComicOrg



from .rawdevart_com_old import RawDevArtComOld


class FirstKissMangaCom(RawDevArtComOld):
    _chapter_selector = r'/manga/[^/]+/chapter-(\d+(?:-\d+)?)'

    def get_files(self):
        parser = self.html_fromstring(self.chapter)
        cls = '.page-break img[data-src],.page-break img[src]'
        return self._images_helper(parser, cls, attr='data-src', alternative_attr='src')


main = FirstKissMangaCom



from .rawdevart_com_old import RawDevArtComOld


class ThreeAsqOrg(RawDevArtComOld):

    def get_chapter_index(self) -> str:
        return self.chapter.split('/')[-2]
    
    def get_files(self):
        parser = self.html_fromstring(self.chapter)
        return self._images_helper(parser, 'img.wp-manga-chapter-img')

    @property
    def chapter(self):
        return super().chapter + '?style=list'


main = ThreeAsqOrg



from manga_py.provider import Provider
from .helpers.std import Std


class MangakuWebId(Provider, Std):

    def get_archive_name(self) -> str:
        ch = self.chapter
        return self.normal_arc_name({'vol': [
            self.chapter_id,
            self.re.search(':[^/]+/([^/]+)', ch).group(1)
        ]})

    def get_chapter_index(self) -> str:
        return str(self.chapter_id)

    def get_content(self):
        return self.http_get(self.get_url())

    def get_manga_name(self) -> str:
        return self._get_name(r'\.\w{2,7}/([^/]+)')

    def get_chapters(self):
        return self._elements('div[style] a[target]')

    def get_files(self):
        content = self.http_get(self.chapter)
        items = self._elements('.entry .separator > a > img', content)
        return [i.get('src') for i in items]

    def get_cover(self) -> str:
        return self._cover_from_content('span > small img')

    def book_meta(self) -> dict:
        # todo meta
        pass


main = MangakuWebId



from os import chmod
from pathlib import Path
from re import compile
from zipfile import ZipFile

from lxml import etree
from requests import get
from selenium.webdriver import Chrome
from selenium.webdriver.chrome.options import Options

from .web_driver import WebDriver, _is_win, _is_mac, _is_linux, UnsupportedOsException


class ChromeDriver(WebDriver):
    url_prefix = 'https://chromedriver.storage.googleapis.com'

    def __init__(self, version: str = None):
        super().__init__((version or '79.0.3945.36'), compile(r'(.+)/'))

    @staticmethod
    def driver_archive() -> str:
        if _is_linux():
            return 'chromedriver_linux64.zip'
        if _is_mac():
            return 'chromedriver_mac64.zip'
        if _is_win():
            return 'chromedriver_win32.zip'
        raise UnsupportedOsException()

    def driver_path(self) -> Path:
        _driver = 'chromedriver'
        if _is_win():
            _driver += '.exe'
        return self.home_path.joinpath(_driver)

    def download_driver(self) -> None:
        path = str(self.home_path.joinpath('driver.zip'))

        with open(path, 'wb') as _driver:
            _driver.write(get('{prefix}/{version}/{driver}'.format(
                prefix=self.url_prefix,
                version=self.driver_version,
                driver=self.driver_archive()
            )).content)
            _driver.close()
        with ZipFile(path) as file:
            _path = str(self.driver_path().parent)
            file.extractall(_path)

    def _make_driver(self) -> None:
        _path = self.driver_path()
        driver_path = str(_path)
        if not _path.is_file():
            self.download_driver()
        _is_win() or chmod(driver_path, 0o755)
        options = Options()
        options.add_argument("--mute-audio")
        options.add_argument('--disable-dev-shm-usage')
        options.add_argument("--disable-blink-features")
        options.add_argument("--disable-blink-features=AutomationControlled")
        options.headless = not self.visible

        self._driver = Chrome(
            executable_path=driver_path,
            options=options,
        )

    def get_available_versions(self) -> set:
        response = get(self.url_prefix)
        xml = etree.fromstring(response.content)
        versions = []
        for child in xml.getchildren():  # type: etree._Element
            for child_child in child.getchildren():  # type: etree._Element
                tag = child_child.tag
                text = child_child.text
                if ~tag.find('Key') and ~text.find('.zip'):
                    versions.append(self._version(text))
        return set(versions)


__all__ = ['ChromeDriver']



from .funmanga_com import FunMangaCom


class MngDoomCom(FunMangaCom):
    def get_files(self):
        content = self.http_get(self.chapter)
        items = self.re.search(r' images = (\[{[^;]+}\])', content)
        if not items:
            return []
        try:
            images = self.json.loads(items.group(1))
            return [i['url'] for i in images]
        except self.json.JSONDecodeError:
            return []


main = MngDoomCom



from manga_py.provider import Provider
from .helpers.std import Std


class MangaSupaCom(Provider, Std):

    def get_chapter_index(self) -> str:
        idx = self.re.search('/chapter_([^/]+)', self.chapter)
        return '-'.join(idx.group(1).split('.'))

    def get_content(self):
        return self._get_content('{}/manga/{}')

    def get_manga_name(self) -> str:
        selector = r'\.\w{2,7}/(?:manga|chapter)/([^/]+)'
        return self._get_name(selector)

    def get_chapters(self):
        return self._elements('.chapter-list .row a')

    def get_files(self):
        parser = self.html_fromstring(self.chapter)
        return self._images_helper(parser, '.vung_doc img')

    def get_cover(self):
        return self._cover_from_content('.info_image img')

    def book_meta(self) -> dict:
        # todo meta
        pass


main = MangaSupaCom



from manga_py.provider import Provider
from .helpers.std import Std


class ReMangaOrg(Provider, Std):

    def get_chapter_index(self) -> str:
        idx = self.re.search(r'/vol(\d+)/(\d+(?:\.\d+)?)', self.chapter)
        return '{}-{}'.format(
            *idx.groups()
        ).replace('.', '-')

    def get_content(self):
        return self._get_content('{}/manga/{}')

    def get_manga_name(self) -> str:
        return self._get_name('/manga/([^/]+)')

    def get_chapters(self):
        return self._elements('.chapter-list a.row')

    def get_files(self):
        n = self.http().normalize_uri
        content = self.http_get(self.chapter)
        match = self.re.search('all_pages = (\[.+?\]);', content).group(1)
        images = self.json.loads(match.replace("'", '"'))
        return [n(i.get('link')) for i in images]

    def get_cover(self) -> str:
        return self._cover_from_content('.item img.head')


main = ReMangaOrg



from manga_py.provider import Provider
from .helpers.std import Std


class MangaFullOrg(Provider, Std):

    def get_chapter_index(self) -> str:
        re = self.re.compile('\w/chapter-(\d+(?:\.\d+)?)')
        return re.search(self.chapter).group(1).replace('.', '-')

    def get_content(self):
        return self._get_content('{}/manga/{}')

    def get_manga_name(self) -> str:
        return self._get_name('/manga/([^/]+)')

    def get_chapters(self):
        return self._elements('.content .chapter-list a')

    def get_files(self):
        parser = self.html_fromstring(self.chapter + '/0')
        return self._images_helper(parser, '.each-page > img')

    def get_cover(self) -> str:
        return self._cover_from_content('.cover-detail > img')


main = MangaFullOrg



from manga_py.provider import Provider
from .helpers.std import Std


class HentaiLxxCom(Provider, Std):
    def get_archive_name(self) -> str:
        return self.chapter[1]

    def get_chapter_index(self) -> str:
        return self.chapter[1]

    def get_content(self):
        return self.http_get(self.get_url())

    def get_manga_name(self) -> str:
        el = self.document_fromstring(self.content, '.title-detail', 0)
        return self.element_text_content(el)

    def get_chapters(self):
        n = self.http().normalize_uri
        return [(n(e.get('href')), self.element_text_content(e)) for e in self._elements('li a.seen')]

    def get_files(self):
        parser = self.html_fromstring(self.chapter[0])
        images = self._images_helper(parser, '#content_chap > .separator > a', 'href')
        if len(images) > 0:
            return images

        return self._images_helper(parser, '#content_chap > div > img')

    def get_cover(self) -> str:
        return self._cover_from_content('#mainpage .row .text-center img[width]')

    def chapter_for_json(self) -> str:
        return self.chapter[0]


main = HentaiLxxCom



from manga_py.provider import Provider
from .helpers.std import Std


class MangaChanMe(Provider, Std):
    _full_name_selector = r'/(?:online|manga|related)/(\d+-.+\.html)'
    _idx_selector = r'/(?:online|manga|related)/(\d+)-'

    def get_chapter_index(self) -> str:
        name = self.chapter
        idx = self.re.search(r'_v(\d+)_ch(\d+)', name).groups()
        return '{}-{}'.format(*idx)

    def get_content(self):
        pass

    def _online_(self, url):
        if self.re.search(r'/online/\d+', url):
            content = self.http_get(url)
            url = self.re.search(r'content_id.+?(/manga/.+\.html)', content).group(1)
        return url

    def get_manga_name(self) -> str:
        _name_selector = r'/(?:online|manga|related)/\d+-(.+)\.html'
        url = self._online_(self.get_url())
        return self.re.search(_name_selector, url).group(1)

    def get_chapters(self):
        url = self._online_(self.get_url())
        url = '{}/manga/{}'.format(
            self.domain,
            self.re.search(self._full_name_selector, url).group(1)
        )
        return self.html_fromstring(url, '.table_cha .manga a')

    def get_files(self):
        content = self.http_get(self.chapter)
        items = self.re.search(r'"?fullimg"?\s?:\s?(\[.+\])', content).group(1)
        images = self.json.loads(items.replace('",]', '"]'))  # patch
        return images

    def get_cover(self):
        selector = r'\.\w{2,7}/[^/]+/(\d+-.+\.html)'
        url = '{}/manga/{}'.format(self.domain, self._get_name(selector))
        img = self._elements('#cover', self.http_get(url))
        if img and len(img):
            return img[0].get('src')

    def book_meta(self) -> dict:
        # todo meta
        pass


main = MangaChanMe



from manga_py.provider import Provider
from .helpers.std import Std


class FunMangaCom(Provider, Std):

    def _get_chapter_idx(self):
        re = self.re.compile(r'\.\w{2,7}/[^/]+/([^/]+)')
        return re.search(self.chapter).group(1)

    def get_chapter_index(self) -> str:
        return self._get_chapter_idx().replace('.', '-')

    def get_content(self):
        return self._get_content('{}/{}')

    def get_manga_name(self) -> str:
        return self._get_name(r'\.\w{2,7}/([^/]+)')

    def get_chapters(self):
        items = self._elements('.chapter-list li > a')
        return [i.get('href') + '/all-pages' for i in items]

    def get_files(self):
        items = self.html_fromstring(self.chapter, '.content-inner > img.img-responsive')
        return [i.get('src') for i in items]

    def get_cover(self):
        return self._cover_from_content('img.img-responsive.mobile-img')

    def prepare_cookies(self):
        self.cf_scrape(self.get_url())


main = FunMangaCom



from manga_py.provider import Provider
from .helpers.std import Std


class WebToonsCom(Provider, Std):
    __titleNo = 0
    __mainUrl = ''
    __next_page_urls = None

    def get_archive_name(self) -> str:
        i = self.re.search(r'\.\w{2,7}%s%s' % (
            r'(?:/|%2F)[^/%]+' * 3,
            r'(?:/|%2F)([^/%]+)',
        ), self.chapter)
        return self.normal_arc_name([self.chapter_id, i.group(1)])

    def get_chapter_index(self) -> str:
        return self.re.search(r'\bepisode_no=(\d+)', self.chapter).group(1)

    def get_content(self):
        return self.http_get(self.__mainUrl)

    def get_manga_name(self) -> str:
        self.__titleNo = self._get_name(r'title_no=(\d+)')
        name = self._get_name(r'\.\w{2,7}/([^/]+/[^/]+/[^/]+)')
        self.__mainUrl = '{}/{}/list?title_no={}'.format(self.domain, name, self.__titleNo)
        return self._get_name(r'\.\w{2,7}/[^/]+/[^/]+/([^/]+)')

    def _chapters(self, content):
        return self._elements('#_listUl li > a', content)

    def _get_page_urls(self, content):
        return self._elements('.paginate a:not([class]):not([onclick])', content)

    def _get_pages_urls(self, content):
        chapters = []
        n = self.http().normalize_uri
        for j in self._get_page_urls(content):  # page-urls
            _content = self.http_get(n(j.get('href')))
            chapters += self._chapters(_content)
        return chapters

    def get_next_page_urls(self, content):
        urls = self._elements('a + a.pg_next', content)
        n = self.http().normalize_uri
        if len(urls):
            self.__next_page_urls.append(n(urls[0].get('href')))
            _content = self.http().get(n(urls[0].get('href')))
            self.get_next_page_urls(_content)

    def get_chapters(self):
        self.log('Parse chapters. Please, wait')
        self.__next_page_urls = []
        chapters = self._chapters(self.content)
        n = self.http().normalize_uri
        chapters += self._get_pages_urls(self.content)  # main page paginator

        self.get_next_page_urls(self.content)
        for url in self.__next_page_urls:
            content = self.http().get(n(url))
            chapters += self._chapters(content)
            chapters += self._get_pages_urls(content)

        return chapters

    def get_files(self):
        parser = self.html_fromstring(self.chapter)
        return self._images_helper(parser, '#_imageList img', 'data-url')

    def get_cover(self) -> str:
        img = self.html_fromstring(self.content, '#content > .detail_bg', 0)
        return self.parse_background(img)

    def prepare_cookies(self):
        self.http().cookies['locale'] = 'en'
        self.http().cookies['needGDPR'] = 'false'
        self.http().cookies['hadSavedCookie'] = 'true'
        self.http().cookies['ageGatePass'] = 'true'
        self.http().cookies['timezoneOffset'] = '+1'

    def allow_auto_change_url(self):
        return False


main = WebToonsCom



from manga_py.provider import Provider
from .helpers.std import Std
from sys import stderr
from manga_py.meta import repo_url


class TmoFansCom(Provider, Std):
    def get_chapter_index(self) -> str:
        try:
            re = self.re.compile(r'Capítulo (\d+(?:\.\d+)?)')
            return re.search(self.chapter[0]).group(1).replace('.', '-')
        except IndexError as e:
            self.log('\nNot found chapter index.\nURL: {}\nChapter: {}\nPlease, report this bug: {}{}\n'.format(
                self.get_url(),
                self.chapter[0],
                repo_url, '/issues/new?template=bug_report.md'
            ), file=stderr)

            raise e

    def get_content(self):
        return self.http_get(self.get_url())

    def get_manga_name(self) -> str:
        return self._get_name(r'/manga/\d+/([^/]+)')

    def get_chapters(self):
        raw_chapters = self._elements('.list-group .upload-link')
        n = self.http().normalize_uri
        re = self.re.compile(r'(.+/)')
        chapters = []
        for i in raw_chapters:
            try:
                text = i.cssselect('.btn-collapse')[0].text_content()
                link = i.cssselect('.list-group .list-group-item .row a')[-1]
            except IndexError:
                self.log('Error extract chapter', file=stderr)
                continue
            request_url = n(link.get('href'))
            response = self.http().requests(request_url, method='head')
            url = n(re.search(response.headers['Location']).group(1))
            chapters.append((
                text,
                url + 'cascade',
            ))

        return chapters

    def get_files(self):
        url = self.chapter[1]
        parser = self.html_fromstring(url)
        return self._images_helper(parser, 'img.viewer-image')

    def get_cover(self) -> str:
        return self._cover_from_content('.book-thumbnail')

    def prepare_cookies(self):
        self.cf_scrape(self.get_url())

    def chapter_for_json(self) -> str:
        return self.chapter[1]


main = TmoFansCom



from .rawdevart_com_old import RawDevArtComOld
from requests import get


class HiperDexCom(RawDevArtComOld):
    _chapter_selector = r'/manga/[^/]+/(?:chapter-)?(\d+(?:[\.-]\d+)?)/'

    def get_chapters(self):
        manga_id = self.document_fromstring(self.content, '#manga-chapters-holder', 0).get('data-id')
        content = self.http_post('{}/wp-admin/admin-ajax.php'.format(self.domain), data={
            'action': 'manga_get_chapters',
            'manga': manga_id,
        })

        return self._elements('.wp-manga-chapter > a', content)

    def _downloader(self, url, file_name):
        with open(str(file_name), 'wb') as w:
            w.write(get(url).content)
        return True

    def prepare_cookies(self):
        self.http()._safe_downloader = self._downloader
        self.http().referer = self.get_url()


main = HiperDexCom



from .mangazuki_me import MangaZukiMe


class MangaKomiCom(MangaZukiMe):

    def get_files(self):
        parser = self.html_fromstring(self.chapter)
        return self._images_helper(
            parser, 'img.wp-manga-chapter-img',
            attr='data-lazy-src', alternative_attr='src'
        )


main = MangaKomiCom



from manga_py.provider import Provider
from .helpers.std import Std


class MangaSaurusCom(Provider, Std):

    def get_chapter_index(self) -> str:
        return str(self.chapter_id)

    def get_content(self):
        return self.http_get('{}/manga/{}/{}'.format(
            self.domain,
            *self.manga_name.split('_')
        ))

    def get_manga_name(self) -> str:
        url = self.get_url()
        if ~url.find('/view/'):
            url = self.html_fromstring(url, '#m_reader_bottom + div > a', 0).get('href')
        result = self.re.search(r'/manga/(\d+)/([^/]+)', url).groups()
        return '{1}_{0}'.format(*result)

    def get_chapters(self):
        return self._elements('.table--chapters td > a')[::-1]

    def __files_helper(self):
        content = self.http_get(self.chapter)
        _path = self.document_fromstring(content, '#imageZone-next > img', 0).get('src')
        path = self.re.search('(http.+?/original)/', _path).group(1) + '/{}/{}-{}{}'
        parser = self.re.search(r'ImageReader\.setImages.+?(\{.+\})', content)
        return path, parser

    def get_files(self):
        path, parser = self.__files_helper()
        if not parser:
            return []
        images = []
        o = self.json.loads(parser.group(1))
        for i in o:
            n = o.get(i)
            _ = n.get('original', {}).get('file', '')
            idx = _.find('.')
            src = path.format(_[:idx], self.manga_name, n['id'], _[idx:])
            images.append(src)
        return images

    def get_cover(self):
        self._cover_from_content('.gallery-info__cover img')

    def book_meta(self) -> dict:
        # todo meta
        pass


main = MangaSaurusCom



from manga_py.provider import Provider
from .helpers.std import Std


class RawDevArtComOld(Provider, Std):
    def get_chapter_index(self) -> str:
        return self.re.search(r'/chapter-(\d+(?:-\d+)?)', self.chapter).group(1)

    def get_content(self):
        return self._get_content('{}/comic/{}/')

    def get_manga_name(self) -> str:
        return self._get_name(r'/comic/([^/]+)')

    def get_chapters(self):
        chapters = []
        chapters += self._elements('.list-group-item > a', self.content)

        for page in self.pages():
            url = '{}/comic/{}/?page={}'.format(self.domain, self.manga_name, page)
            chapters += self._elements('.list-group-item > a', self.http_get(url))

        return chapters

    def pages(self):
        pages = self._elements('.pagination .page-item > a')

        try:
            max_page = self.re.search(r'page=(\d+)', pages[-1].get('href')).group(1)
            return map(str, range(2, 1 + int(max_page)))
        except IndexError:
            return []

    def get_files(self):
        parser = self.html_fromstring(self.chapter)
        return self._images_helper(parser, 'img.img-fluid', 'data-src', 'src')

    def get_cover(self) -> str:
        parser = self.document_fromstring(self.content)
        return self._images_helper(parser, 'img.img-fluid')[0]


main = RawDevArtComOld



from manga_py.provider import Provider
from .helpers.std import Std


class MangaMexatCom(Provider, Std):

    def get_chapter_index(self) -> str:
        return self.chapter[1].replace('.', '-')

    def get_content(self):
        return self._get_content('{}/category/{}/')

    def get_manga_name(self) -> str:
        return self._get_name('/category/([^/]+)')

    def get_chapters(self):
        items = self._elements('.content .entry td + td > a')
        return [(i.get('href', i.text_content().strip())) for i in items]

    def _get_img(self, parser):
        return self._images_helper(parser, '.pic > a > img')

    def get_files(self):
        parser = self.html_fromstring(self.chapter)
        pages = self._first_select_options(parser, '#manga_pid', True)
        images = self._get_img(parser)
        for p in pages:
            url = self.chapter + '?pid=' + p.get('value')
            parser = self.html_fromstring(url)
            images += self._get_img(parser)
        return images

    def get_cover(self) -> str:
        return self._cover_from_content('.archive-meta img')

    def book_meta(self) -> dict:
        pass

    def chapter_for_json(self):
        return self.chapter[0]


main = MangaMexatCom



from manga_py.provider import Provider
from .helpers.std import Std


class MangabbCo(Provider, Std):

    def get_chapter_index(self) -> str:
        chapter = self.chapter
        idx = chapter.rfind('/chapter-')
        return chapter[1 + idx:]

    def get_content(self):
        return self._get_content('{}/manga/{}')

    def get_manga_name(self) -> str:
        return self._get_name(r'\.\w{2,7}/(?:manga/)?([^/]+)')

    def get_chapters(self):
        content = self.document_fromstring(self.content, '#chapters a')
        if not content:
            return []
        selector = '#asset_1 select.chapter_select > option'
        result = self.html_fromstring(content, selector)
        return [i.get('value') for i in result[::-1]]

    @staticmethod
    def __get_img(parser):
        return parser.cssselect('#manga_viewer > a > img')[0].get('src')

    @staticmethod
    def _img_lifehack1(img, pages_list, images):
        n = 1
        for i in pages_list:
            n += 1
            images.append('{}{}{}'.format(img[0], n, img[1]))

    def _img_lifehack2(self, pages_list, images):
        for page in pages_list:
            parser = self.html_fromstring(page)
            images.append(self.__get_img(parser))

    def get_files(self):
        parser = self.html_fromstring(self.chapter, '#body', 0)
        result = parser.cssselect('#asset_2 select.page_select option + option')
        pages_list = [i.get('value') for i in result]
        _first_image = self.__get_img(parser)
        images = [_first_image]

        img = self.re.search(r'(.+/)\d(\.\w{2,7})', _first_image)
        if img:  # livehack
            self._img_lifehack1(img.groups(), pages_list, images)
        else:
            self._img_lifehack2(pages_list, images)

        return images

    def get_cover(self):
        return self._cover_from_content('#series_image')

    def book_meta(self) -> dict:
        # todo meta
        pass


main = MangabbCo



from .helpers.std import Std
from .pecintakomik_com import PecintaKomikCom


class PecintaKomikComManga(PecintaKomikCom, Std):

    def get_content(self):
        return self._get_content('{}/manga/{}/')

    def get_manga_name(self) -> str:
        return self._get_name(r'/manga/([^/]+)')

    def get_chapters(self):
        parser = self.document_fromstring(self.content)
        items = self._first_select_options(parser, 'select[name="chapter"]', False)
        url = '{}/manga/{}/%s/full'.format(
            self.domain,
            self.manga_name
        )
        return [url % i.get('value') for i in items]

    def get_cover(self) -> str:
        pass


main = PecintaKomikComManga



from manga_py.provider import Provider
from .helpers.std import Std


class KomikCastCom(Provider, Std):
    def get_chapter_index(self) -> str:
        re = self.re.compile(r'-chapter-(\d+(?:-\d+)?)')
        return re.search(self.chapter).group(1)

    def get_content(self):
        return self._get_content('{}/{}')

    def get_manga_name(self) -> str:
        url = self.get_url()
        if ~url.find('/chapter/'):
            url = self.html_fromstring(url, '.allc a', 0).get('href')
            self._params['url'] = self.http().normalize_uri(url)
            return self.get_manga_name()
        return self._get_name(r'\.com/([^/]+)')

    def get_chapters(self) -> list:
        return self._elements('.mangainfo .leftoff a')

    def get_files(self) -> list:
        parser = self.html_fromstring(self.chapter)
        return self._images_helper(parser, '#readerarea img')

    def get_cover(self):
        return self._cover_from_content('.topinfo img')

    def book_meta(self) -> dict:
        # todo meta
        pass


main = KomikCastCom



from manga_py.provider import Provider
from .helpers.std import Std


class SevenSamaCom(Provider, Std):

    def get_archive_name(self) -> str:
        self._vol_fill = True
        name = self.re.sub('[^a-zA-Z0-9]+', '_', self.chapter['chapter_name'])
        return self.normal_arc_name([
            self.chapter['number'],
            str(self.chapter_id),
            name
        ])

    def get_chapter_index(self) -> str:
        return self.chapter_id

    def get_content(self):
        return self._get_content('{}/manga/{}')

    def get_manga_name(self) -> str:
        return self._get_name('/manga/([^/]+)')

    def get_chapters(self):
        idx = self.re.search(r'/manga/.+?/(\d+)', self.get_url()).group(1)
        chapters = []
        for i in range(1, 1000):
            content = self.http_get('{}/series/chapters_list.json?page={}&id_serie={}'.format(
                self.domain, i, idx
            ), {'x-requested-with': 'XMLHttpRequest'})
            data = self.json.loads(content)
            if data['chapters'] is False:
                break
            chapters += self.__prepare_chapters(data['chapters'])
        return chapters

    @staticmethod
    def __prepare_chapters(items):
        chapters = []
        for i in items:
            for k, j in i['releases'].items():
                chapter = i.copy()
                chapter['release'] = j
                chapter['release_id'] = k
                chapters.append(chapter)
        return chapters

    def get_files(self):
        url = self.chapter_for_json()
        content = self.http_get('{}{}'.format(self.domain, url))
        api_key = self.re.search(r'this\.page\.identifier\s*=\s*[\'"](.+)[\'"]', content).group(1)
        url = '{}/leitor/pages/{}.json?key={}'.format(
            self.domain, self.chapter['release']['id_release'], api_key
        )
        images = self.json.loads(self.http_get(url, {'x-requested-with': 'XMLHttpRequest'}))
        return images['images']

    def get_cover(self) -> str:
        return self._cover_from_content('.cover img.cover')

    def book_meta(self) -> dict:
        pass

    def chapter_for_json(self) -> str:
        return self.chapter['release']['link']


main = SevenSamaCom



from .gomanga_co import GoMangaCo


class LoliVaultNet(GoMangaCo):
    _name_re = '/online/[^/]+/([^/]+)/'
    _content_str = '{}/online/series/{}/'
    _chapters_selector = '.list .group .element .title a'


main = LoliVaultNet



import re
from typing import List, Union
import json


__all__ = ['AcQqComCrypt', 'AcQqComCrypt26']


_site_key = "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789+/="
RE_EXTRACT = re.compile(r'"url":\s*"([^"]+?\.(?:jpg|jpeg|png|webp)[^"]*?)"')
RE_PARTS = re.compile(r'(\d+[a-zA-Z]+)')
RE_INT = re.compile(r'^(\d+)')


def key(data: str, idx: int):
    try:
        char = data[idx]
    except IndexError:
        return 0

    position = _site_key.find(char)
    return position


def _decode(data) -> List[str]:
    data = re.sub(r'[^A-Za-z0-9+/=\\]', '', data)
    a = []
    e = 0
    while e < len(data):
        b = key(data, e)
        e += 1
        d = key(data, e)
        e += 1
        f = key(data, e)
        e += 1
        g = key(data, e)
        e += 1

        b = b << 2 | d >> 4
        d = (d & 15) << 4 | f >> 2
        h = (f & 3) << 6 | g
        a.append(chr(b))

        if f != 64:
            a.append(chr(d))

        if g != 64:
            a.append(chr(h))

    return a


class AcQqComCrypt:
    __slots__ = []

    def decode(self, data):
        return self._sub_dict(''.join(_decode(data)))

    @staticmethod
    def _sub_dict(data):
        try:
            data = re.search(r'picture.*?(\[{.+}\])', data).group(1)
            return json.loads(data)
        except Exception as e:
            return {}


class AcQqComCrypt26:  # v2.6
    __slots__ = []

    @staticmethod
    def decode(data: str) -> str:
        return ''.join(_decode(data))

    @staticmethod
    def remap_content(nonce: str, content: Union[list, str]) -> str:
        _data = list(content)
        parts = RE_PARTS.findall(nonce)
        parts_len = len(parts)

        _replace = re.compile(r'\d+')

        while parts_len > 0:
            parts_len -= 1

            _locate = int(RE_INT.search(parts[parts_len]).group(1)) & 255
            _str = _replace.sub('', parts[parts_len])

            _data = _data[0:_locate] + _data[_locate + len(_str):]

        return ''.join(_data)

    # @staticmethod
    # def extract_images(content: str):
    #     content = content.replace(r'\"', '"').replace(r'\\', '\\')
    #     images = RE_EXTRACT.findall(content)
    #     return [i.replace(r'\/', '/') for i in images]



from manga_py.crypt.base_lib import BaseLib
from manga_py.provider import Provider
from .helpers.std import Std


class ToonKorCo(Provider, Std):

    def get_archive_name(self) -> str:
        return self.normal_arc_name(self.get_chapter_index())

    def get_chapter_index(self) -> str:
        re = self.re.compile(r'\.\w{2,7}/[^_]+_(.+)\.html?')
        return re.search(self.chapter).group(1)

    def get_content(self):
        return self._get_content('{}/{}')

    def get_manga_name(self) -> str:
        return self._get_name(r'\.\w{2,7}/([^_]+)')

    def get_chapters(self):
        items = self._elements('#fboardlist td.episode__index')
        n = self.http().normalize_uri
        return [n(i.get('data-role')) for i in items]

    def get_files(self):
        content = self.http_get(self.chapter)
        imgs = self.re.search(r'toon_img\s=\s["\'](.+?)["\']', content)
        if not imgs:
            return []
        content = BaseLib.base64decode(imgs.group(1)).decode()
        n = self.http().normalize_uri
        return [n(i.get('src')) for i in self._elements('img', content)]

    def get_cover(self) -> str:
        return self._cover_from_content('.bt_thumb a img')

    def book_meta(self) -> dict:
        pass


main = ToonKorCo



from threading import Thread


class MultiThreads:
    threads = None
    max_threads = 4
    to_run = None

    def __init__(self):
        self.threads = []
        self.to_run = []

    def add(self, target: callable, args: tuple):
        self.threads.append(Thread(target=target, args=args, daemon=True))

    def _run_processes(self, callback: callable = None, n: int = None):
        for t in self.to_run:
            if not n:
                t.join()
                callback is not None and callback()

    def start(self, callback: callable = None):
        for n, t in enumerate(self.threads):  # starting all threads
            t.start()
            self.to_run.append(t)
            self._run_processes(callback, (n + 1) % self.max_threads)
        self._run_processes(callback)
        self.threads = []



"""
manga-py module for CLI and its options.
"""

import os
from argparse import ArgumentParser, MetavarTypeHelpFormatter

from manga_py.meta import version
from ._args_debug import _args_debug
from ._args_downloading import _args_downloading
from ._args_general import _args_general
from ._args_image import _args_image
from ._args_reader import _args_reader


class Formatter(MetavarTypeHelpFormatter):
    def __init__(self, prog, indent_increment=2, max_help_position=30, width=None):
        if width is None:
            try:
                width = int(os.environ['COLUMNS'])
            except (KeyError, ValueError):
                width = 100
        super().__init__(prog, indent_increment, max_help_position, width)

    def _format_action_invocation(self, action):
        if not action.option_strings:
            default = self._get_default_metavar_for_positional(action)
            metavar, = self._metavar_formatter(action, default)(1)
            return metavar
        else:
            parts = []
            if action.nargs == 0:
                parts.extend(action.option_strings)
            else:
                default = self._get_default_metavar_for_optional(action)
                args_string = self._format_args(action, default)
                args_len = len(action.option_strings) - 1
                for i, option_string in enumerate(action.option_strings):
                    if i == args_len:
                        parts.append('%s %s' % (option_string, args_string))
                    else:
                        parts.append(option_string)
            return ', '.join(parts)


def get_cli_arguments() -> ArgumentParser:  # pragma: no cover
    """
    Method to generate manga-py CLI with its options.
    """
    args_parser = ArgumentParser(
        add_help=False,
        formatter_class=Formatter,
        prog="manga-py",
        description=(
            '%(prog)s is the universal manga downloader (for your offline reading).\n  '
            'Site: https://manga-py.com/manga-py/\n  '
            'Source-code: https://github.com/manga-py/manga-py\n  '
            'Version: ' + version
        ),
        epilog=(
            'So, that is how %(prog)s can be executed to download yours favourite mangas.\n'
            'Enjoy! 😉'
        )
    )

    _args_general(args_parser, version)
    _args_downloading(args_parser)
    _args_reader(args_parser)
    _args_image(args_parser)
    _args_debug(args_parser)

    return args_parser



from manga_py.provider import Provider
from .helpers.std import Std


class ReadMangaEu(Provider, Std):

    def get_chapter_index(self) -> str:
        idx = self.re.search('/manga/\d+/[^/]+/([^/]+)', self.chapter)
        return '-'.join(idx.group(1).split('.'))

    def get_content(self):
        name = self._get_name('/(manga/\d+/[^/]+)')
        return self.http_get('{}/{}'.format(self.domain, name))

    def get_manga_name(self) -> str:
        return self._get_name('/manga/\d+/([^/]+)')

    def get_chapters(self):
        selector = '#chapters_b a[href*="/manga/"]'
        return self._elements(selector)

    def parse_files(self, parser):
        images_class = '.mainContent img.ebook_img'
        return self._images_helper(parser, images_class)

    def get_files(self):
        parser = self.html_fromstring(self.chapter)
        pages = parser.cssselect('#jumpto > option + option')
        images = self.parse_files(parser)
        for i in pages:
            url = self.http().normalize_uri(i.get('value'))
            parser = self.html_fromstring(url)
            images += self.parse_files(parser)
        return images

    def get_cover(self):
        return self._cover_from_content('.ebook_cover')

    def book_meta(self) -> dict:
        # todo meta
        pass


main = ReadMangaEu



from urllib.parse import unquote_plus

from manga_py.provider import Provider
from .helpers.std import Std


class MangaXNet(Provider, Std):
    __name = None

    def get_chapter_index(self) -> str:
        re = self.re.compile(r'\.\w{2,7}/\w/[^/]+/([^/]+)')
        return re.search(self.chapter).group(1).replace('.', '-')

    def get_content(self):
        url = '{}/m/{}'.format(
            self.domain,
            self.__name,
        )
        return self.http_get(url)

    def get_manga_name(self) -> str:
        self.__name = self._get_name(r'\.\w{2,7}/\w/([^/]+)')
        return unquote_plus(self.__name)

    def get_chapters(self):
        return self._elements('.chlist li a')

    def get_files(self):
        ch = self.re.sub(r'(\.\w{2,7})/\w/', r'\1/f/', self.chapter)
        parser = self.html_fromstring(ch)
        return self._images_helper(parser, 'img.center-block')

    def get_cover(self) -> str:
        return self._cover_from_content('.thumbnail > img')

    def book_meta(self) -> dict:
        pass


main = MangaXNet



from .gomanga_co import GoMangaCo
from .helpers.std import Std


class RiceBallIciousInfo(GoMangaCo, Std):
    _name_re = '/fs/reader/[^/]+/([^/]+)/'
    _content_str = '{}/fs/reader/series/{}/'


main = RiceBallIciousInfo



from .rawdevart_com_old import RawDevArtComOld


class ManhwaClub(RawDevArtComOld):
    _chapter_selector = r'/chapter-(\d+(?:\.\d+)?)'

    def get_content(self):
        return self._get_content('{}/manhwa/{}')

    def get_manga_name(self) -> str:
        return self._get_name('/manhwa/([^/]+)')


main = ManhwaClub



from .senmanga_com import SenMangaCom


class RawSenmangaCom(SenMangaCom):

    def get_archive_name(self) -> str:
        return self.get_chapter_index()

    def get_chapter_index(self):
        ch = self.chapter
        re = r'\.\w{2,7}/[^/]+/([^/]+)'
        return self.re.search(re, ch)


main = RawSenmangaCom



from .mangahub_io import MangaHubIo


class MangaReaderSite(MangaHubIo):
    __manga_id = None
    __graphql = 'https://api2.mangahub.io/graphql?query={}'

    def get_chapter_index(self) -> str:
        return self.chapter_for_json().replace('.', '-')

    def get_chapters(self):
        chapters = self.json.loads(self.http_get(self.__graphql.format(
            '{manga(x:mr01,slug:"%s"){chapters{id,number,title,slug,date}}}' % self.manga_name
        )))
        return chapters.get('data', {}).get('manga', {}).get('chapters', [])[::-1]

    def get_files(self):
        data = self.http_get(self.__graphql.format(
            '{chapter(x:mr01,slug:"%s",number:%d){pages}}' % (self.manga_name, self.chapter['number'])
        ))
        images = self.json.loads(data)
        pages = self.json.loads(images.get('data', {}).get('chapter', {}).get('pages', '{}'))
        return ['https://cdn.mangahub.io/file/imghub/%s' % pages[i] for i in pages]

    def chapter_for_json(self) -> str:
        return str(self.chapter['number'])


main = MangaReaderSite



from manga_py.provider import Provider
from .helpers.std import Std


class MangaHeavenClub(Provider, Std):
    def get_chapter_index(self) -> str:
        ch = self.re.search(r'-chapter-(\d+(?:\.\d+)?)', self.chapter)
        return ch.group(1).replace('.', '-')

    def get_content(self):
        return self._get_content('{}/read-manga/{}')

    def get_manga_name(self) -> str:
        return self._get_name(r'/read-manga/([^/]+)')

    def get_chapters(self):
        return self._elements('.chapter > a')

    def get_files(self):
        parser = self.html_fromstring('%s/0' % self.chapter)
        return self._images_helper(parser, '.page-chapter > img')

    def get_cover(self) -> str:
        return self._cover_from_content('.detail-info img')


main = MangaHeavenClub



from manga_py.provider import Provider
from .helpers.std import Std


class OtakuSmashCom(Provider, Std):
    selector = r'https?://[^/]+/(comics/|read-\w+/|reader/)?([^/]+)'
    prefix = '/'

    def get_chapter_index(self) -> str:
        selector = self.selector + '/([^/]+)'
        idx = self.re.search(selector, self.chapter)
        return '-'.join(idx.group(3).split('.'))

    def get_content(self):
        return self.http_get(self._get_manga_url())

    def get_manga_name(self) -> str:
        result = self.re.search(self.selector, self.get_url())
        self.prefix = result.group(1)
        return result.group(2)

    def get_chapters(self):
        parser = self.document_fromstring(self.content)
        items = self._first_select_options(parser, '.pager select[name="chapter"]', False)
        url = self._get_manga_url()
        return ['{}{}/'.format(url, i.get('value')) for i in items]

    def get_files(self):
        chapter = self.chapter
        parser = self.html_fromstring(chapter)
        pages = self._first_select_options(parser, '.mid .pager select[name="page"]')
        images = []
        _img = self._get_image(parser)
        _img and images.append(_img)
        self.log('Get pages... Please, wait')
        for page in pages:
            parser = self.html_fromstring('{}{}/'.format(chapter, page.get('value')))
            _img = self._get_image(parser)
            _img and images.append(_img)
        return images

    def _get_manga_url(self):
        return '{}/{}{}/'.format(self.domain, self.prefix, self.manga_name)

    def _get_image(self, parser):
        image = parser.cssselect('a > img.picture')
        if not len(image):
            return False
        image = image[0].get('src')
        if image[0] == '/':
            return self.http().normalize_uri(image)
        base_uri = parser.cssselect('base')
        if len(base_uri):
            base_uri = base_uri[0].get('href')
        else:
            base_uri = self.chapter
        return base_uri + image

    def get_cover(self):
        pass  # TODO

    def book_meta(self) -> dict:
        # todo meta
        pass


main = OtakuSmashCom



from manga_py.provider import Provider
from .helpers.std import Std


class MangaSh(Provider, Std):
    _api_url = 'https://api.manga.sh/api/v1/'
    _cdn_url = 'https://cdn.manga.sh/'

    def get_chapter_index(self) -> str:
        chapter = self.chapter
        _ch = chapter.get('ChapterNumberAbsolute', self.chapter_id)
        _vol = chapter.get('VolumeNumber', 0)
        _ch_v = chapter.get('ChapterNumberVolume', '')
        if _ch_v:
            _ch_v = '_' + _ch_v
        return '{}-{}{}'.format(_vol, _ch, _ch_v)

    def get_content(self):
        idx = self._get_name(r'/comics/(\d+)')
        url = '{}series_chapters?query=SeriesId.Id:{}&order=asc&sortby=TimeUploaded&limit=0&offset=0'
        content = self.http_get(url.format(self._api_url, idx))
        return self.json.loads(content)

    def get_manga_name(self) -> str:
        content = self.content.get('response')[0]
        return content.get('SeriesId').get('Name')

    def get_chapters(self):
        return list(self.content.get('response', []))

    def _url_helper(self, chapter):
        return '{}series_chapters/{}'.format(
            self._api_url,
            chapter.get('Hash')
        )

    def get_files(self):
        url = self._url_helper(self.chapter)
        items = self.json.loads(self.http_get(url))
        items = items.get('response', [{}])[0].get('SeriesChaptersFiles', {})
        return [self._cdn_url + i.get('Name') for i in items]

    def get_cover(self) -> str:
        content = self.content.get('response')[0]
        content = content.get('SeriesId').get('CoverImage')
        return '{}/covers/{}'.format(self._cdn_url, content)

    def book_meta(self) -> dict:
        # todo meta
        pass

    def chapter_for_json(self):
        return self._url_helper(self.chapter)


main = MangaSh



# cli chapters parser
class ChapterHelper:
    chapters = ''

    def __init__(self, chapters: str):
        self.chapters = chapters
        if isinstance(self.chapters, str):
            self.chapters = self.chapters.split(' ')

    def get_chapters(self, urls):
        chapters = []
        for i, url in enumerate(urls):
            if i in self.chapters:
                chapters.append(urls)
        return chapters



from manga_py.provider import Provider
from .helpers.std import Std


class ChoChoxCom(Provider, Std):

    def get_archive_name(self) -> str:
        return 'archive'

    def get_chapter_index(self) -> str:
        return '0'

    def get_content(self):
        return self._get_content('{}/{}')

    def get_manga_name(self) -> str:
        return self._get_name(r'\.\w{2,7}/([^/]+)')

    def get_chapters(self):
        return [b'']

    def get_files(self):
        return [i.get('src') for i in self._elements('img.alignnone')]

    def get_cover(self) -> str:
        pass

    def book_meta(self) -> dict:
        # todo meta
        pass

    def chapter_for_json(self):
        return self.get_url()


main = ChoChoxCom



#!/usr/bin/python3
# -*- coding: utf-8 -*-

from atexit import register as atexit_register
from json import dumps
from logging import info, warning, basicConfig, INFO, WARN, getLogRecordFactory, setLogRecordFactory
from os import makedirs, path
from shutil import rmtree
from sys import exit
from .cli.args import get_cli_arguments

from .meta import version
from .fs import get_temp_path, root_path
from .base_classes.web_driver import get_display, get_driver

from .fs import get_info
from .cli import Cli
from .info import Info
import better_exceptions
better_exceptions.hook()


@atexit_register
def before_shutdown():
    get_display() and get_display().stop()
    get_driver() and get_driver().close()
    temp_dir = get_temp_path()
    path.isdir(temp_dir) and rmtree(temp_dir)


def _run_util(args):
    """
    :param args:
    :return:
    :rtype Info|str
    """
    parse_args = args.parse_args()
    _info = Info(parse_args)

    _info.start()
    Cli(args, _info).start()

    if parse_args.print_json:
        _info = dumps(
            _info.get(),
            indent=2,
        )
    else:
        _info = []

    return _info


def _update_all(args):
    parse_args = args.parse_args()
    info('Update all')
    multi_info = {}

    dst = parse_args.destination
    json_info = get_info(dst)

    for i in json_info:
        parse_args.manga_name = i['manga_name']
        parse_args.url = i['url']
        _info = _run_util(args)
        multi_info[i['directory']] = _info
    parse_args.quiet or (parse_args.print_json and print(multi_info))


def run_util(args):
    """
    Main util function
    :param args:
    :return:
    """

    temp_path = get_temp_path()
    path.isdir(temp_path) or makedirs(temp_path)

    parse_args = args.parse_args()

    try:
        _info = _run_util(args)
        parse_args.quiet or (parse_args.print_json and print(_info))
        return 0
    except KeyboardInterrupt:
        warning('\nUser interrupt')
        return 1


__root = len(root_path())
__old_factory = getLogRecordFactory()


def __log_factory(*args, **kwargs):
    record = __old_factory(*args, **kwargs)
    record.my_pathname = record.pathname[__root:]
    record.custom_attribute = "my-attr"
    return record


def main():
    args = get_cli_arguments()
    setLogRecordFactory(__log_factory)

    log_format = '"%(levelname)s:%(my_pathname)s:%(lineno)s:%(asctime)s:%(message)s"'
    basicConfig(level=(INFO if args.parse_args().debug else WARN), format=log_format)

    if ~version.find('alpha'):
        warning('Alpha release! There may be errors!')

    exit(run_util(args))


if __name__ == '__main__':
    main()



from manga_py.provider import Provider
from .helpers.std import Std


class ComicWebNewTypeCom(Provider, Std):

    def get_chapter_index(self) -> str:
        re = self.re.compile('/contents/[^/]+/([^/]+)')
        return re.search(self.chapter).group(1)

    def get_content(self):
        return self._get_content('{}/contents/{}/')

    def get_manga_name(self) -> str:
        return self._get_name('/contents/([^/]+)')

    def get_chapters(self):
        return self._elements('#episodeList li.ListCard a')

    def get_files(self):
        url = self.chapter
        items = self.http_get(url + 'json/', headers={'x-requested-with': 'XMLHttpRequest'})
        imgs = self.json.loads(items)
        imgs = [self.re.sub(r'jpg.+', 'jpg', img) for img in imgs]
        return imgs

    def get_cover(self) -> str:
        return self._cover_from_content('.WorkSummary-content img')

    def book_meta(self) -> dict:
        # todo meta
        pass


main = ComicWebNewTypeCom



from manga_py.provider import Provider
from .helpers.std import Std


class MangaReaderXyz(Provider, Std):

    def get_chapter_index(self) -> str:
        re = self.re.compile(r'/chapter-(\d+(?:\.\d+)?)')
        return re.search(self.chapter).group(1).replace('.', '-')

    def get_content(self):
        return self._get_content('{}/manga/{}')

    def get_manga_name(self) -> str:
        if ~self.get_url().find('/manga/'):
            re = '/manga/([^/]+)'
        else:
            re = '/([^/]+)/chapter-'
        return self._get_name(re)

    def get_chapters(self):
        return self._elements('.table td > a,.chapter-list div.row a')

    def get_files(self):
        parser = self.html_fromstring(self.chapter + '/0')
        return self._images_helper(parser, '#view-chapter img,#vungdoc img')

    def get_cover(self) -> str:
        return self._cover_from_content('img.img-thumbnail,.manga-info-pic > img')

    def book_meta(self) -> dict:
        pass


main = MangaReaderXyz



from logging import info, error

__all__ = ['get_display', 'make_driver']

__cache = {
    'display': None,
    'driver': None,
}


def get_display():
    return __cache['display']


def get_driver():
    return __cache['driver']


def make_driver(browser: str = None):
    if __cache['driver'] is not None:
        return __cache['driver']

    _assert_selenium()

    __cache['display'] = _init_display()

    if browser is None:
        __cache['driver'] = _auto_browser()
        return __cache['driver']

    if browser == 'chrome':
        __cache['driver'] = _chrome(True)
        return __cache['driver']

    if browser == 'firefox':
        __cache['driver'] = _firefox(True)
        return __cache['driver']

    raise RuntimeError('Bad driver type')


def _assert_selenium():
    try:
        from selenium.webdriver import __version__
    except ImportError as e:
        error('Selenium not installed. Please, run "pip install selenium"')
        raise e


def _init_display():
    try:
        from pyvirtualdisplay import Display
        _display = Display(visible=False, size=(1920, 1080))
        _display.start()
    except ImportError:
        _display = None
        info('Use real display. See here: https://github.com/ponty/PyVirtualDisplay/blob/master/README.rst')
    return _display


def _auto_browser():
    try:
        _driver = _chrome()
        info('Use chrome')
        return _driver
    except Exception as ce:
        try:
            _driver = _firefox()
            info('Use _firefox')
            return _driver
        except Exception as e:
            error('Browser driver init error', ce, e)
            raise e


def _chrome(show_error: bool = False):
    try:
        from .chrome import ChromeDriver
        return ChromeDriver().init_driver()
    except Exception as e:
        show_error and error('Chrome browser driver init error', e)
        raise e


def _firefox(show_error: bool = False):
    try:
        from .firefox import FirefoxDriver
        return FirefoxDriver().init_driver()
    except Exception as e:
        show_error and error('Firefox browser driver init error', e)
        raise e



from .read_powermanga_org import ReadPowerMangaOrg


class TripleSevenScansCom(ReadPowerMangaOrg):
    _name_re = '/reader/[^/]+/([^/]+)/'
    _content_str = '{}/reader/series/{}/'


main = TripleSevenScansCom



from .gomanga_co import GoMangaCo


class GMangaMe(GoMangaCo):
    _name_re = '/mangas/([^/]+)'
    _content_str = '{}/mangas/{}'
    _chapters_selector = 'a.chapter-link'

    def get_chapter_index(self) -> str:
        selector = r'/mangas/[^/]+/(\d+/[^/]+)'
        idx = self.re.search(selector, self.chapter).group(1)
        return idx.replace('/', '-')

    def _get_json_selector(self, content):
        return r'1:\salphanumSort\((\[.+\])\)'

    def get_cover(self) -> str:
        image = self.re.search(r'"image"\s?:\s?"(.+)",', self.content)
        if image:
            return image.group(1)


main = GMangaMe



from argparse import ArgumentParser

from .info import Info
from .providers import get_provider
from .provider import Provider

from requests import get
from logging import warning


class Parser:
    params = None
    provider = None

    def __init__(self, args):
        self.params = {}
        self.args = args
        self._add_params(args)

    def _add_params(self, params: ArgumentParser = None):
        if params is None:
            params = self.args.parse_args()
        else:
            params = params.parse_args()
        self.params = params.__dict__

    def init_provider(
            self,
            chapter_progress: callable = None,
            global_progress: callable = None,
            log: callable = None,
            quest: callable = None,
            info: Info = None,
            quest_password: callable = None,
    ):

        original_url = self.params.get('url', '')
        provider_url = self.params.get('force_provider', None)
        provider = get_provider(provider_url or original_url)

        if isinstance(provider, bool):
            raise AttributeError('Provider not found')

        # update url (if redirect)
        self.provider = provider(info)  # type: Provider

        self.provider.original_url = original_url

        real_url = self.check_url(original_url)

        if self.provider.allow_auto_change_url():
            if real_url != original_url:
                warning('Manga url changed! New url: {}'.format(real_url))
            self.params['url'] = real_url

        self.provider.quiet = self.params.get('quiet', False)

        self.provider.set_chapter_progress_callback(chapter_progress)
        self.provider.set_global_progress_callback(global_progress)
        self.provider.set_log_callback(log)
        self.provider.set_quest_callback(quest)
        self.provider.set_quest_password_callback(quest_password)

    def start(self):
        self.provider.process(self.params['url'], self.params)

    def check_url(self, url):
        proxy = self.params.get('proxy', None)

        with get(url, stream=True, proxies=({
                'http': proxy,
                'https': proxy,
            } if proxy else None)) as response:
            _url = response.url
            if url != _url:
                url = _url
        return url




from time import sleep

from requests import get

from .gomanga_co import GoMangaCo
from .helpers.std import Std


class LHTranslationCom(GoMangaCo, Std):
    _name_re = r'/(?:truyen|manga)-([^/]+)\.html'
    _content_str = '{}/manga-{}.html'
    _chapters_selector = '#tab-chapper td > a.chapter,#list-chapters a.chapter'

    def get_chapter_index(self) -> str:
        idx = self.re.search(r'-chapter-(.+?)\.html', self.chapter)
        return idx.group(1).replace('.', '-')

    def get_files(self):
        content = self.http_get(self.chapter)
        parser = self.document_fromstring(content, 'article#content,.chapter-content', 0)
        return self._images_helper(parser, 'img.chapter-img', 'data-original')


main = LHTranslationCom




def _args_downloading(args_parser):  # pragma: no cover
    args = args_parser.add_argument_group('Downloading options')

    args.add_argument(
        '-s',
        '--skip-volumes',
        metavar='COUNT',
        type=int,
        help=(
            'Skip a total number, i.e. %(metavar)s, of volumes.'
        ),
        default=0
    )

    args.add_argument(
        '-c',
        '--max-volumes',
        metavar='COUNT',
        type=int,
        default=0,
        help=(
            'Download a maximum number, i.e. %(metavar)s, of volumes. '
            'E.g.: `--max-volumes 2` will download at most 2 volumes. '
            'If %(metavar)s is `0` (zero) then it will download all available volumes.'
        )
    )

    args.add_argument(
        '-u',
        '--user-agent',
        type=str,
        help=(
            'Set an user-agent. '
            # 'Don\'t work from protected sites.'
        )
    )

    args.add_argument(
        '--cookies',
        type=str,
        help=(
            'Set specified cookies. Example: --cookies cf_clearance=9c.. test=5a..'
        ),
        nargs='*',
    )

    args.add_argument(
        '-x',
        '--proxy',
        type=str,
        help=(
            'Set a http proxy.'
        )
    )

    args.add_argument(
        '-r',
        '--reverse-downloading',
        action='store_true',
        help=(
            'Download manga volumes in a reverse order. '
            'By default, manga is downloaded in ascendent order '
            '(i.e. volume 00, volume 01, volume 02...). '
            'If `--reverse-downloading` is actived, then manga is downloaded in descendent order '
            '(i.e. volume 99, volume 98, volume 97...).'
        )
    )

    args.add_argument(
        '-w',
        '--rewrite-exists-archives',
        action='store_true',
        help=(
            '(Re)Download manga volume if it already exists locally in the directory destination. '
            'Your manga files can be overwrited, so be careful.'
        )
    )

    args.add_argument(
        '-t',
        '--max-threads',
        type=int,
        default=4,
        help=(
            'Set the maximum number of threads, i.e. MAX_THREADS, to be avaliable to manga-py. '
            'Threads run in pseudo-parallel when execute the process to download the manga images.'
        )
    )

    args.add_argument(
        '-f',
        '--zero-fill',
        action='store_true',
        help=(
            'Pad a `-0` (dash-and-zero) at right for all downloaded manga volume filenames. '
            'E.g. from `vol_001.zip` to `vol_001-0.zip`. '
            'It is useful to standardize the filenames between: '
            '1) normal manga volumes (e.g. vol_006.zip) and, '
            '2) abnormal manga volumes (e.g. vol_006-5.zip). '
            'An abnormal manga volume is a released volume like: '
            'extra chapters, '
            'bonuses, '
            'updated, '
            'typos corrected, '
            'spelling errors corrected; '
            'and so on.'
        )
    )

    args.add_argument(
        '-p',
        '--force-provider',
        metavar='URL',
        type=str,
        help='Force use specific provider',
    )

    args.add_argument(
        '-N',
        '--with-manga-name',
        action='store_true',
        help=(
            'Pad the manga name at left for all downloaded manga volumes filenames. '
            'E.g. from `vol_001.zip` to `manga_name-vol_001.zip`.'
        )
    )

    args.add_argument(
        '-o',
        '--override-archive-name',
        metavar='ARCHIVE_NAME',
        type=str,
        default='',
        dest='override_archive_name',
        help=(
            'Pad %(metavar)s at left for all downloaded manga volumes filename. '
            'E.g from `vol_001.zip` to `%(metavar)s-vol_001.zip`.'
        )
    )

    args.add_argument(
        '-S',
        '--min-free-space',
        metavar='MB',
        type=int,
        default=100,
        help=(
            'Alert when the minimum free disc space, i.e. MB, is reached. '
            'Insert it in order of megabytes (Mb).'
        )
    )

    args.add_argument(
        '--skip-incomplete-chapters',
        action='store_true',
        help=(
            'Incomplete chapters are not written to disk instead of saving them as "IMAGES_SKIP_ERROR"'
        )
    )




import json
import os
import re
from abc import ABC
from logging import info, warning, error
from os import path
from sys import stderr
from typing import Tuple
from .base_classes.comic_info_builder import *
from PIL import Image

from .base_classes import (
    Abstract,
    Archive,
    Base,
    Callbacks,
    cf_scrape,
    Static,
    ArchiveName,
)
from .fs import (
    get_temp_path,
    is_file,
    basename,
    remove_file_query_params,
    path_join,
    file_size,
)
from .info import Info
from .meta import repo_url, version
from .download_methods import OnePerOneDownloader


class Provider(Base, Abstract, Static, Callbacks, ArchiveName, ABC):
    _volumes_count = 0
    _archive = None
    _zero_fill = False
    _with_manga_name = False
    _info = None
    _simulate = False
    _volume = None
    _show_chapter_info = False
    _save_chapter_info = False
    _save_manga_info = False
    _debug = False
    _override_name = ''
    _downloader = OnePerOneDownloader
    global_progress = None

    __images_cache = []

    def __init__(self, info: Info = None):
        super().__init__()
        self.re = re
        self.json = json
        self._params['temp_directory'] = get_temp_path()
        self._info = info

    def _params_parser(self, params):
        # image params
        self._set_if_not_none(self._image_params, 'crop_blank', params.get('crop_blank', False))
        self._set_if_not_none(
            self._image_params, 'crop',
            (params.get('xt', 0),
             params.get('xr', 0),
             params.get('xb', 0),
             params.get('xl', 0)),
        )
        self._image_params['no_webp'] = params.get('no_webp', False)
        # downloading params
        self._set_if_not_none(self._params, 'destination', params.get('destination', None))
        self._zero_fill = params.get('zero_fill')
        self._with_manga_name = params.get('with_manga_name')
        self._simulate = params.get('simulate')
        self._show_chapter_info = params.get('show_chapter_info', False)
        self._save_chapter_info = params.get('save_chapter_info', False)
        self._save_manga_info = params.get('save_manga_info', False)
        self._debug = params.get('debug', False)
        self._override_name = self._params.get('override_archive_name')
        if self._with_manga_name and self._override_name:
            raise RuntimeError('Conflict of parameters. Please use only --with-manga-name, or --override-archive-name')
        self._fill_arguments(params.get('arguments') or [])
        self._skip_incomplete_chapters = params.get('skip_incomplete_chapters', False)

    def process(self, url, params=None):  # Main method
        self.prepare_download(url, params)
        self.loop_chapters()

    def prepare_download(self, url, params=None):
        self._params['url'] = url
        params = params if isinstance(params, dict) else {}
        self._params_parser(params)
        for i in params:
            self._params.setdefault(i, params[i])

        proxy = params.get('proxy', None)
        if proxy is not None:
            self._storage['proxies'] = {
                'http': proxy,
                'https': proxy,
            }

        if self.__manual_ua():
            self.update_ua(self._params['user_agent'])

            cookies = (c.split('=', 1) for c in self._params['cookies'])
            self.update_cookies({c[0]: c[1] for c in cookies})

        self.prepare_cookies()

        info('Manga name: %s' % self.manga_name)
        info('Content length %d' % len(self.content))
        self.chapters = self._prepare_chapters(self.get_chapters())
        info('Chapters received (%d)' % len(self.chapters))

        if not self._params.get('reverse_downloading', False):
            self.chapters = self._storage['chapters'][::-1]

        self._storage['init_cookies'] = self._storage['cookies']

        __ua = self.http().user_agent

        self._info.set_ua(__ua)

        info('User-agent: "%s"' % __ua)

        if self._save_manga_info:
            details = self.manga_details()
            if details is not None:
                manga_info_path = path.abspath(path.join(self.get_archive_path()[0], os.pardir))
                path.isdir(manga_info_path) or os.makedirs(manga_info_path)

                with open(path.join(manga_info_path, 'info.json'), 'w') as manga_info_file:
                    manga_info_file.write(json.dumps(self.manga_details()))

            else:
                warning('No manga details was found!')
                warning('Possibly the provider has not yet been implemented to get this information')

    def _min_max_calculate(self):
        nb_chapters = len(self.chapters)
        _min = self._params.get('skip_volumes', 0)
        _max = self._params.get('max_volumes', 0)
        # Beware, 0 can also come from command line param
        _max = _max if _max else nb_chapters
        _max = min(nb_chapters, _max + _min)
        self.chapters_count = _max - _min
        return _min, _max

    def loop_chapters(self):
        _min, _max = self._min_max_calculate()
        count = 0  # count downloaded chapters
        for idx, __url in enumerate(self.chapters[:_min], start=1):
            info('Skip chapter %d / %s' % (idx, __url))

        dl = self._downloader(self)

        if callable(self.global_progress):
            self.global_progress(self.chapters_count, 0, True)

        for idx, __url in enumerate(self.chapters[_min:_max], start=_min + 1):
            self.chapter_id = idx - 1
            if dl.already_downloaded():
                info('Skip chapter %d / %s' % (idx, __url))
                continue
            if self._show_chapter_info:
                print('\n\nCurrent chapter url: %s\n' % (self.chapter,), file=stderr)

            count += 1

            chapter_for_json = self.chapter_for_json()

            chapter = chapter_for_json if chapter_for_json is not None else self.chapter

            _path = '%s.%s' % self.get_archive_path()

            self._info.add_volume(chapter, _path)

            if not self._simulate:
                self.before_download_chapter()
                dl.volume = chapter
                dl.download_chapter(self.chapter, self.get_archive_path())
                self.after_download_chapter()

            if callable(self.global_progress):
                self.global_progress(self.chapters_count, idx - _min)
            info('Processed chapter %d / %s' % (idx, __url))

        for idx, __url in enumerate(self.chapters[_max:], start=_max + 1):
            info('Skip chapter %d / %s' % (idx, __url))

        if count == 0 and not self.quiet:
            print('No new chapters found', file=stderr)

    def get_archive_path(self) -> Tuple[str, str]:
        if self._override_name:
            _path = "{}_{}".format(self._override_name, str(self.normal_arc_name(self.get_chapter_index().split('-'))))
        else:
            # see Std
            _path = remove_file_query_params(self.get_archive_name())
        _path = self.remove_not_ascii(_path)

        if not _path:
            _path = str(self.chapter_id)

        additional_data_name = ''
        if self.http().has_error:
            additional_data_name = '.ERROR'
            self.http().has_error = False
            warning('Error processing chapter.')

        # Manga online biz use this naming scheme (see http2). Not sure if wanted
        # arc_name =  '{:0>3}-{}'.format(idx, self.get_archive_name())
        # If we want to keep it, maybe instead override self.get_archive_name ?
        arc_name = '%s%s' % (_path, additional_data_name)

        return (
            path_join(
                self._params.get('destination', 'Manga'),
                self.name,
                arc_name
            ).replace('?', '_').replace('"', '_').replace('>', '_').replace('<', '_').replace('|', '_')  # Windows...
            , self._archive_type()
        )

    def html_fromstring(self, url, selector: str = None, idx: int = None):
        params = {}
        if isinstance(url, dict):
            params = url['params']
            url = url['url']
        return self.document_fromstring(self.http_get(url, **params), selector, idx)

    def cf_scrape(self, url):
        """
        WARNING! Thins function replace cookies!
        :param url: str
        :return:
        """
        try:
            params = cf_scrape(url)
            if len(params):
                self.update_cookies(params[0])
                self.update_ua(params[1])
                self._params['cf-protect'] = True
        except Exception:
            if not self.__manual_ua():
                self.log('Please, use --cookie and --user-agent options')

    def __manual_ua(self) -> bool:
        return self._params['cookies'] and len(self._params['cookies']) and self._params['user_agent'] and len(
            self._params['user_agent'])

    def update_ua(self, ua):
        self._storage['user_agent'] = ua
        self.http().user_agent = ua
        self._info and self._info.set_ua(ua)

    def update_cookies(self, cookies):
        for k in cookies:
            self._storage['cookies'][k] = cookies[k]
            self.http().cookies[k] = cookies[k]

    def before_download_file(self, idx, url):
        url = self.before_file_save(url, idx)
        filename = remove_file_query_params(basename(url))
        _path = Static.remove_not_ascii(self._image_name(idx, filename))
        _path = get_temp_path(_path)
        return _path, idx, url

    # region specified data for eduhoribe/comic-builder (see https://github.com/manga-py/manga-py/issues/347)

    def chapter_details(self, chapter) -> dict:
        """
        Following the pattern specified in
        https://github.com/eduhoribe/comic-builder/blob/goshujin-sama/samples/chapter-metadata-sample.json
        """
        pass

    def manga_details(self) -> dict:
        """
        Following the pattern specified in
        https://github.com/eduhoribe/comic-builder/blob/goshujin-sama/samples/comic-metadata-sample.json
        """
        pass

    # endregion



import re
import importlib

providers_list = {
    '1stkissmanga_com': [
        r'1stkissmanga\.com/manga/.',
    ],
    '_3asq_org': [
        r'3asq\.org/.',
    ],
    '7sama_com': [
        r'7sama\.com/manga/.',
    ],
    '18comic_org': [
        r'18comic\.org/album/\d'
    ],
    'ac_qq_com': [
        r'ac\.qq\.com/Comic.+?/id/\d',
    ],
    'acomics_ru': [
        r'acomics\.ru/~.',
    ],
    'adulto_seinagi_org': [
        # r'adulto\.seinagi\.org/(series|read)/.',
        r'xanime-seduccion\.com/(series|read)/.',
        # r'twistedhelscans\.com/(series|read)/.',
        r'reader\.evilflowers\.com/(series|read)/.',
    ],
    'allhentai_ru': [
        r'allhentai\.ru/.',
    ],
    'animextremist_com': [
        r'animextremist\.com/mangas-online/.',
    ],
    'antisensescans_com': [
        r'antisensescans\.com/online/(series|read)/.',
    ],
    'asmhentai_com': [
        r'asmhentai\.com/(g|gallery)/\d',
    ],
    'atfbooru_ninja': [
        r'atfbooru\.ninja/posts.',
    ],
    'bato_to': [
        r'bato\.to/series/\d',
        r'mangawindow\.net/series/\d',
    ],
    'blogtruyen_com': [
        r'blogtruyen\.com/.',
    ],
    'cdmnet_com_br': [
        r'cdmnet\.com\.br/titulos/.',
        r'centraldemangas.online/titulos/.'
    ],
    'chochox_com': [
        r'chochox\.com/.',
    ],
    'comicextra_com': [
        r'comicextra\.com/.',
    ],
    'comic_webnewtype_com': [
        r'comic\.webnewtype\.com/contents/.',
    ],
    'comico_jp': [
        r'comico\.jp/(?:challenge/)?(detail|articleList).+titleNo.',
    ],
    'comicvn_net': [
        r'comicvn\.net/truyen-tranh-online/.',
    ],
    'cycomi_com': [
        r'cycomi\.com/fw/cycomibrowser/chapter/title/\d',
    ],
    'danbooru_donmai_us': [
        r'danbooru\.donmai\.us/posts.',
    ],
    'darkskyprojects_org': [
        r'darkskyprojects\.org/biblioteca/.',
    ],
    'dejameprobar_es': [
        r'menudo-fansub\.com/slide/.',
        r'yuri-ism\.net/slide/.',
    ],
    'desu_me': [
        r'desu\.me/manga/.',
    ],
    'doujins_com': [
        r'doujins\.com/hentai-manga/.',
        r'doujins\.com/gallery/.',
        r'doujin-moe\.us/gallery/.',
    ],
    'e_hentai_org': [
        r'e-hentai\.org/g/\d',
    ],
    'fanfox_net': [
        r'fanfox\.net/manga/.',
    ],
    'funmanga_com': [
        r'funmanga\.com/.',
    ],
    'gmanga_me': [
        r'gmanga\.me/mangas/.',
    ],
    'gomanga_co': [
        r'kobato\.hologfx\.com/reader/.',
    ],
    'helveticascans_com': [
        r'helveticascans\.com/r/(series|read)/.',
    ],
    # 'hatigarmscans_eu': [
    #     r'hatigarmscans\.net/hs/(series|read).',
    #     r'hatigarmscans\.net/manga/.',
    # ],
    'heavenmanga_biz': [
        r'heavenmanga\.com/.',
    ],
    'hentai2read_com': [
        r'hentai2read\.com/.',
    ],
    'hentai_cafe': [
        r'hentai\.cafe/.',
    ],
    'hentai_chan_me': [
        r'hentai-chan\.me/(related|manga|online)/.',  # todo
    ],
    'hentai_image_com': [
        r'hentai-img\.com/image/.',
        r'hentai-image\.com/image/.',
    ],
    'hentaihand_com': [
        r'hentaihand\.com/comic/\d',
    ],
    'hentaifox_com': [
        r'hentaifox\.com/.',
    ],
    'hentaihere_com': [
        r'hentaihere\.com/m/.',
    ],
    'hentailxx_com': [
        r'hentailxx\.com/story/view\.php\?id=\d'
    ],
    'hentainexus_com': [
        r'hentainexus\.com/(read|view)/\d'
    ],
    'hentaiporns_net': [
        r'hentaiporns\.net/.'
    ],
    'hentairead_com': [
        r'hentairead\.com/.',
    ],
    'hentaivn_net': [
        r'hentaivn\.net/\d+-doc-.',
    ],
    'hitomi_la': [
        r'hitomi\.la/(galleries|reader)/.',
    ],
    'hgamecg_com': [
        r'hgamecg\.com/index/category/\d',
    ],
    'hiperdex_com': [
        r'hiperdex\.com/manga/.',
    ],
    'hitmanga_eu': [
        r'mymanga\.io/.',
    ],
    'hocvientruyentranh_com': [
        r'hocvientruyentranh\.com/(manga|chapter)/.',
    ],
    'hotchocolatescans_com': [
        # r'mangaichiscans\.mokkori\.fr/fs/(series|read)/.',
        r'taptaptaptaptap\.net/fs/(series|read)/.',
    ],
    # 'riceballicious_info': [
    #     r'riceballicious\.info/fs/reader/(series|read)/.',
    # ],
    'rocaca_com': [
        r'rocaca\.com/manga/.',
    ],
    'inmanga_com': [
        r'inmanga\.com/ver/manga/.',
    ],
    'isekaiscan_com': [
        r'isekaiscan\.com/manga/.',
    ],
    'jaiminisbox_com': [
        r'jaiminisbox\.com/reader/series/.',
    ],
    'japscan_com': [
        r'japscan\.(cc|com|co|to|se)/.',
    ],
    'jurnalu_ru': [
        r'jurnalu\.ru/online-reading/.',
    ],
    'kissmanga_com': [
        r'kissmanga\.com/Manga/.',
    ],
    'komikcast_com': [
        r'komikcast\.com/.',
    ],
    'komikid_com': [
        r'komikid\.com/manga/.',
        r'mangazuki\.co/manga/.',
        r'mangadoor\.com/manga/.',
        r'cmreader\.info/manga/.',
        r'rawmangaupdate\.com/manga/.',
        r'manga-v2\.mangavadisi\.org/manga/.',
        r'universoyuri\.com/manga/.',
        r'digitalteam1\.altervista\.org/manga/.',
        r'komikgue\.com/manga/.',
        r'onma\.me/manga/.',
    ],
    'kumanga_com': [
        r'kumanga\.com/manga/\d',
    ],
    # 'lector_kirishimafansub_com': [
    #     r'lector\.kirishimafansub\.com/(lector/)?(series|read)/.',
    # ],
    'leitor_net': [
        r'leitor\.net/manga/.',
    ],
    'leviatanscans_com': [
        r'leviatanscans\.com/comics/\d',
        r'es\.leviatanscans\.com/comics/\d',
        r'krakenscans\.com/comics/\d',
        r'skscans\.com/comics/\d',
    ],
    'lhtranslation_com': [
        r'read\.lhtranslation\.com/(truyen|manga)-.',
        r'lhtranslation\.net/(truyen|manga)-.',
    ],
    'lolibooru_moe': [
        r'lolibooru\.moe/post.',
    ],
    'lolivault_net': [
        r'lolivault\.net/online/(series|read).',
    ],
    'luscious_net': [
        r'luscious\.net/.+/album/.',
        r'luscious\.net/albums/.',
    ],
    'manga41_com': [
        r'manga41\.com/manga/.',
    ],
    'manga_ae': [
        r'mangaae\.com/.',
    ],
    'manga_mexat_com': [
        r'manga\.mexat\.com/category/.',
    ],
    'manga_online_biz': [
        r'manga-online\.biz/.',
    ],
    'manga_tube_me': [
        r'manga-tube\.me/series/.',
    ],
    'mangaarabteam_com': [
        r'mangaarabteam\.com/.',
    ],
    'manga_tr_com': [
        r'manga-tr\.com/(manga|id)-.',
    ],
    'mangabat_com': [
        r'mangabat\.com/(manga|chapter)/.',
        r'mangabat\.com/read-.',
    ],
    # 'mangabb_co': [  removed #306
    #     r'mangabb\.co/.',
    # ],
    'mangabox_me': [
        r'mangabox\.me/reader/.',
    ],
    'mangacanblog_com': [
        r'mangacanblog\.com/.',
    ],
    'mangaclub_ru': [
        r'mangaclub\.ru/.',
    ],
    'mangadeep_com': [
        r'mangadeep\.com/.',
        r'manga99\.com/.',
    ],
    # 'mangadex_org': [
    'mangadex_org_v2': [
        r'mangadex\.org/(manga|title)/.',
    ],
    'mangaeden_com': [
        r'mangaeden\.com/[^/]+/[^/]+-manga/.',
        r'perveden\.com/[^/]+/[^/]+-manga/.',
    ],
    'mangafreak_net_download': [
        r'mangafreak\.net/Manga/.',
    ],
    # 'mangafull_org': [
    #     r'mangafull\.org/manga/.',
    # ],
    # 'mangaheaven_club': [
    #     r'mangaheaven\.club/read-manga/.',
    # ],
    # 'mangaheaven_xyz': [
    #     r'mangaheaven\.xyz/manga/.',
    # ],
    'mangahere_cc': [
        r'mangahere\.cc/manga/.',
    ],
    'mangaid_me': [
        # r'mangaid\.co/manga/.',
        # r'mangaid\.net/manga/.',
        # r'mangaid\.me/manga/.',
        r'bacamanga\.co/manga/.',
    ],
    'mangahome_com': [
        r'mangahome\.com/manga/.',
    ],
    'mangahub_io': [
        r'mangahub\.io/(manga|chapter)/.',
        r'mangakakalot\.fun/(manga|chapter)/.',
        r'mangahere\.onl/(manga|chapter)/.',
    ],
    'mangahub_ru': [
        r'mangahub\.ru/.',
    ],
    'mangaindo_web_id': [
        r'mangaindo\.web\.id/.',
    ],
    'mangainn_net': [
        r'mangainn\.net/.',
    ],
    'mangakakalot_com': [
        r'mangakakalot\.com/manga/.',
        r'mangakakalot\.com/read-.',
    ],
    'mangakatana_com': [
        r'mangakatana\.com/manga/.',
    ],
    'mangakomi_com': [
        r'mangakomi\.com/manga/.',
    ],
    'mangaku_web_id': [
        r'mangaku\.in/.',
        r'mangaku\.pro/.',
    ],
    'mangalib_me': [
        r'mangalib\.me/.',
    ],
    'mangalife_us': [
        r'mangalife\.us/(read-online|manga)/.',
        r'manga4life\.com/(read-online|manga)/.',
    ],
    'mangamew_com': [
        r'mangamew\.com/(\w+-)?manga/.',
    ],
    'mangamew_com_vn': [
        r'mangamew\.com/(\w+-)?truyen/.',
    ],
    'manganelo_com': [
        r'manganelo\.com/(manga|chapter)/.',
    ],
    'manganelos_com': [
        r'manganelos\.com/manga/.',
    ],
    'mangaonline_com_br': [
        r'mangaonline\.com\.br/.',
    ],
    # 'mangaonlinehere_com': [
    #     r'mangaonlinehere\.com/(manga-info|read-online)/.',
    # ],
    'mangapanda_com': [
        r'mangapanda\.com/.',
    ],
    'mangapark_me': [
        r'mangapark\.net/manga/.',
    ],
    'mangareader_net': [
        r'mangareader\.net/.',
    ],
    'mangareader_site': [
        r'mangareader\.site',
    ],
    # 'mangareader_xyz': [
    #     r'mangareader\.xyz/manga/.',
    #     r'mangareader\.xyz/.+?/chapter-\d',
    # ],
    'mangarussia_com': [
        r'mangarussia\.com/(manga|chapter)/.',
    ],
    # 'mangasaurus_com': [
    #     r'mangasaurus\.com/(manga|view).',
    # ],
    'mangaseeonline_us': [
        # r'mangaseeonline\.us/(read-online|manga)/.',
        r'mangasee123\.com/(read-online|manga)/.',
    ],
    'mangashiro_net': [
        r'mangashiro\.net/.',
    ],
    # 'mangasupa_com': [
    #     r'mangasupa\.com/(manga|chapter)/.',
    # ],
    'mangasushi_net': [
        r'mangasushi\.net/manga/.',
        r'asurascans\.com/manga/.',
    ],
    'mangatail_com': [
        r'mangatail\.me/(manga|chapter|node|content)/.',
    ],
    'mangatown_com': [
        r'mangatown\.com/manga/.',
    ],
    # 'mangatrue_com': [
    #     r'mangatrue\.com/manga/.',
    #     r'mangaall\.com/manga/.',
    # ],
    'mangawindow_net': [
        r'mangawindow\.net/(series|chapter)/\d',  # is different!
    ],
    # 'mangax_net': [
    #     r'mangax\.net/\w/.',
    # ],
    'mangazuki_me': [
        r'mangazuki\.me/manga/.',
        r'mangazuki\.online/mangas/.',
    ],
    'manhuagui_com': [
        r'manhuagui\.com/comic/\d',
    ],
    'manhuatai_com': [
        r'manhuatai\.com/.',
    ],
    'manhwa18_net': [
        r'manhwa18\.net/manga-.',
    ],
    'manhwa_club': [
        r'manhwa\.club/manhwa/.',
    ],
    'manhwareader_com': [
        r'manhwareader\.com/manga/.',
    ],
    'merakiscans_com': [
        r'merakiscans\.com/manga/.',
    ],
    # 'mngcow_co': [
    #     r'mngcow\.co/.',
    # ],
    # 'mngdoom_com': [
    #     r'mangadoom\.co/.',
    #     r'mngdoom\.com/.',
    # ],
    'mymangalist_org': [
        r'mymangalist.org/(read|chapter)-',
    ],
    'myreadingmanga_info': [
        r'myreadingmanga\.info/.',
    ],
    'neumanga_tv': [
        r'neumanga\.tv/manga/.',
    ],
    'nhentai_net': [
        r'nhentai\.net/g/.',
    ],
    'niadd_com': [
        r'niadd\.com/manga/.',
    ],
    'nightow_net': [
        r'nightow\.net/online/\?manga=.',
    ],
    'nineanime_com': [
        r'nineanime\.com/manga/.+\.html'
    ],
    'ninemanga_com': [
        r'ninemanga\.com/(manga|chapter).',
        r'addfunny\.com/(manga|chapter).',
    ],
    'noranofansub_com': [
        r'noranofansub\.com(/lector)?/(series/|read/)?.',
    ],
    'nozominofansub_com': [  # mangazuki_co
        r'godsrealmscan\.com/public(/index\.php)?/manga/.',
    ],
    'otakusmash_com': [
        #r'otakusmash\.com/.',  removed #306
        # r'mrsmanga\.com/.',
        r'mentalmanga\.com/.'
        r'omgbeaupeep\.com/comics/.',
    ],
    'otscans_com': [
        r'otscans\.com/foolslide/(series|read)/.',
    ],
    # 'pecintakomik_com_manga': [
    #     r'pecintakomik\.com/manga/.',
    # ],
    # 'pecintakomik_com': [
    #     r'pecintakomik\.com/.',
    # ],
    'plus_comico_jp_manga': [
        r'plus\.comico\.jp/manga/\d',
    ],
    'plus_comico_jp': [
        r'plus\.comico\.jp/store/\d',
    ],
    'porncomix_info': [
        r'porncomix\.info/.',
    ],
    # 'psychoplay_co': [
    #     r'psychoplay\.co/(series|read)/.',
    # ],
    'puzzmos_com': [
        r'puzzmos\.com/manga/.',
    ],
    r'pururin_io': [
        r'pururin\.io/(gallery|read)/.',
    ],
    'pzykosis666hfansub_com': [
        r'pzykosis666hfansub\.com/online/.',
    ],
    'ravens_scans_com': [
        r'ravens-scans\.com(/lector)?/(serie/|read/).',
    ],
    'raw_senmanga_com': [
        r'raw\.senmanga\.com/.',
    ],
    'rawdevart_com': [
        r'rawdevart\.com/comic/.',
    ],
    'rawlh_com': [
        r'lhscan\.net/(truyen|manga|read)-.',
        # r'rawqq\.com/(truyen|manga|read)-.',
        r'kisslove\.net/(truyen|manga|read)-.',  # r'rawqv\.com/(truyen|manga|read)-.',
        r'loveheaven\.net/(truyen|manga|read)-.',
    ],
    # 'rawneko_com': [
    #     r'rawneko\.com/manga/.',
    # ],
    'read_egscans_com': [
        r'read\.egscans\.com/.',
    ],
    'read_powermanga_org': [
        r'read\.powermanga\.org/(series|read)/.',
        r'reader\.kireicake\.com/(series|read)/.',
        r'slide\.world-three\.org/(series|read)/.',
        r'reader\.thecatscans\.com/(series|read)/.',
        r'reader\.deathtollscans\.net/(series|read)/.',
        r'reader\.jokerfansub\.com/(series|read)/.',
        r'lector\.patyscans\.com/(series|read)/.',
        r'truecolorsscans\.miocio\.org/(series|read)/.',
        r'reader\.letitgo\.scans\.today/(series|read)/.',
        r'reader\.fos-scans\.com/(series|read)/.',
        r'reader\.serenade\.moe/(series|read)/.',
        r'reader\.vortex-scans\.com/(series|read)/.',
        r'reader\.silentsky-scans\.net/(series|read)/.',
        r'reader\.manga-download\.org/(series|read)/.',
    ],
    'read_ptscans_com': [
        r'read\.ptscans\.com/series/.'
    ],
    'read_yagami_me': [
        r'read\.yagami\.me/series/\w',
    ],
    'comicpunch_net_manga': [
        r'comicpunch\.net/asiancomics/.',
    ],
    'comicpunch_net': [
        r'comicpunch\.net/.',
    ],
    # 'reader_championscans_com': [
    #     r'reader\.championscans\.com/(series|read)/.',
    # ],
    'readcomiconline_to': [
        r'readcomiconline\.to/Comic/.',
    ],
    'readcomicsonline_ru': [
        r'readcomicsonline\.ru/comic/.',
    ],
    'readmanga_me': [
        r'readmanga\.live/.',
        r'mintmanga\.live/.',
        r'selfmanga\.ru/.',
    ],
    'readmng_com': [
        r'readmng\.com/.',
    ],
    # 'readms_net': [
    #     r'readms\.net/(r|manga)/.',
    # ],
    # todo #266
    'remanga_org': [
        r'remanga\.org/manga/.',
    ],
    'santosfansub_com': [
        r'santosfansub\.com/Slide/.',
    ],
    'senmanga_com': [
        r'senmanga\.com/.',
    ],
    'serimanga_com': [
        r'serimanga.com/manga/.'
    ],
    'shakai_ru': [
        r'shakai\.ru/manga.*?/\d',
    ],
    'shogakukan_co_jp': [
        r'shogakukan\.co\.jp/books/\d',
        r'shogakukan\.co\.jp/magazines/series/\d',
    ],
    # 'shogakukan_tameshiyo_me': [
    #     r'shogakukan\.tameshiyo\.me/\d',
    # ],
    'siberowl_com': [
        r'siberowl\.com/mangas/.',
    ],
    'sleepypandascans_co': [
        r'sleepypandascans\.co/(Series|Reader)/.',
    ],
    'subapics_com': [
        r'mangakita\.net/manga/.',
        r'mangakita\.net/.+-chapter-.',
        r'komikstation\.com/manga/.',
        r'komikstation\.com/.+-chapter-.',
        r'mangavy\.com/manga/.',
        r'mangavy\.com/.+-chapter-.',
    ],
    'submanga_online': [
        r'submanga\.online/manga/.',
    ],
    'sunday_webry_com': [
        r'sunday-webry\.com/series/\d',
    ],
    'taadd_com': [
        r'taadd\.com/(book|chapter)/.',
    ],
    'tenmanga_com': [
        r'tenmanga\.com/(book|chapter)/.',
    ],
    'translate_webtoons_com': [
        r'translate\.webtoons\.com/webtoonVersion\?webtoonNo.',
    ],
    'tonarinoyj_jp': [
        r'tonarinoyj\.jp/episode/.',
    ],
    'triplesevenscans_com': [
        r'sensescans\.com/reader/(series|read)/.',
        # r'triplesevenscans\.com/reader/(series|read)/.',
    ],
    'truyen_vnsharing_site': [
        r'truyen\.vnsharing\.site/index/read/.',
    ],
    'truyenchon_com': [
        r'truyenchon\.com/truyen/.',
        r'nettruyen\.com/truyen-tranh/.',
    ],
    'truyentranhtuan_com': [
        r'truyentranhtuan\.com/.',
    ],
    'tsumino_com': [
        r'tsumino\.com/Book/Info/\d',
        r'tsumino\.com/Read/View/\d',
    ],
    'viz_com': [
        r'viz\.com/shonenjump/chapters/.',
    ],
    'web_ace_jp': [
        r'web-ace\.jp/youngaceup/contents/\d',
    ],
    'webtoons_com': [
        r'webtoons\.com/[^/]+/[^/]+/.',
    ],
    'westmanga_info': [
        r'westmanga\.info/.',
    ],
    'whitecloudpavilion_com': [
        r'whitecloudpavilion\.com/manga/free/manga/.',
    ],
    'wiemanga_com': [
        r'wiemanga\.com/(manga|chapter)/.',
    ],
    'wmanga_ru': [
        r'wmanga\.ru/starter/manga_.',
    ],
    'yande_re': [
        r'yande\.re/post.',
    ],
    'zeroscans_com': [
        r'zeroscans\.com/comics/.',
        r'reaperscans\.com/comics/.',
        r'secretscans\.co/comics/.',
        r'kkjscans\.co/comics/.',
        r'kkjscans\.co/comics/.',
        r'edelgardescans\.com/comics/.',
        r'hunlight-scans\.info/comics/.',
        r'hatigarmscanz\.net/comics/.',
        r'the-nonames\.com/comics/.',
        r'methodscans\.com/comics/.',
        r'leviatanscans\.com/comics/.'
    ],
}


def __check_provider(provider, url):
    items = [r'\b' + i for i in provider]
    reg = '(?:' + '|'.join(items) + ')'
    return re.search(reg, url)


def get_provider(url):
    fromlist = 'manga_py.providers'
    for i in providers_list:
        if __check_provider(providers_list[i], url):
            provider = importlib.import_module('%s.%s' % (fromlist, i))
            return provider.main
    return False



from manga_py.provider import Provider
from .helpers.std import Std


class HentaiVnNet(Provider, Std):
    def get_chapter_index(self) -> str:
        return self.chapter_id

    def get_content(self):
        return self.http().get(self.get_url())

    def get_manga_name(self) -> str:
        return self._get_name(r'\.\w{2,7}/\d+(?:-\w+){2}-(.+)\.htm')

    def get_chapters(self):
        return self._elements('.listing a')

    def prepare_cookies(self):
        self.update_cookies({'page_ads_right': '1'})
        self.cf_scrape(self.get_url())

    def get_files(self):
        content = self.http_get(self.chapter)
        parser = self.document_fromstring(content)
        return self._images_helper(parser, '#image img')

    def get_cover(self) -> str:
        return self._cover_from_content('.page-ava img')


main = HentaiVnNet



from manga_py.provider import Provider
from .helpers.std import Std


class MangaHomeCom(Provider, Std):

    def get_chapter_index(self) -> str:
        selector = r'/manga/[^/]+/[^\d]+(\d+)(?:\.(\d+))?'
        idx = self.re.search(selector, self.chapter).groups()
        return self._join_groups(idx)

    def get_content(self):
        return self._get_content('{}/manga/{}')

    def get_manga_name(self) -> str:
        return self._get_name('/manga/([^/]+)')

    def get_chapters(self):
        return self._elements('.detail-chlist a')

    def get_files(self):
        n = self.http().normalize_uri
        img_selector = 'img#image'
        _url = n(self.chapter)
        parser = self.html_fromstring(_url)
        p_selector = '.mangaread-top .mangaread-pagenav select'
        pages = self._first_select_options(parser, p_selector)
        images = self._images_helper(parser, img_selector)
        for i in pages:
            parser = self.html_fromstring(n(i.get('value')))
            images += self._images_helper(parser, img_selector)
        return images

    def get_cover(self):
        return self._cover_from_content('.detail-cover')

    def book_meta(self) -> dict:
        # todo meta
        pass


main = MangaHomeCom



from manga_py.provider import Provider
from .helpers.std import Std


class ManhwaCo(Provider, Std):

    def get_chapter_index(self) -> str:
        chapter = self.chapter
        return self.re.search(r'\.\w{2,7}/Reader/[^/]+/([^/]+)', chapter).group(1)

    def get_content(self):
        return self._get_content('{}/Series/{}')

    def get_manga_name(self) -> str:
        return self._get_name(r'\.\w{2,7}/(?:Series|Reader)/([^/]+)')

    def get_chapters(self):
        return self._elements('.list-group .list-group-item')

    def get_files(self):
        content = self.http_get(self.chapter)
        parser = self.document_fromstring(content)
        return self._images_helper(parser, 'img.img-fluid')

    def get_cover(self) -> str:
        return self._cover_from_content('img.card-img-top')

    def book_meta(self) -> dict:
        # todo meta
        pass


main = ManhwaCo



from typing import Callable


class Callbacks:
    def _set_default_if_not_callable(self, attr_name: str, callback: Callable, default: Callable):
        cb = callback if callable(callback) else default
        setattr(self, attr_name, cb)

    def set_quest_callback(self, callback: Callable):  # Required call from initiator (CLI, GUI)
        self._set_default_if_not_callable('quest', callback, self.quest)

    def set_chapter_progress_callback(self, callback: Callable):  # Required call from initiator (CLI, GUI)
        self._set_default_if_not_callable('chapter_progress', callback, self.progress)

    def set_global_progress_callback(self, callback: Callable):  # Required call from initiator (CLI, GUI)
        self._set_default_if_not_callable('global_progress', callback, self.progress)

    def set_log_callback(self, callback: Callable):  # Required call from initiator (CLI, GUI)
         self._set_default_if_not_callable('log', callback, self.progress)

    def set_quest_password_callback(self, callback: Callable):  # Required call from iterator (CLI, GUI)
        self._set_default_if_not_callable('quest_password', callback, self.progress)

    def quest(self, *args, **kwargs):
        pass

    def quest_password(self, *args, **kwargs):
        pass

    def progress(self, *args, **kwargs):
        pass

    def log(self, *args, **kwargs):
        pass

    def book_meta(self) -> dict:
        return {}



from .helpers.nine_manga import NineHelper
from .helpers.std import Std


class NineMangaCom(NineHelper, Std):
    _local_storage = None

    def get_chapter_index(self) -> str:
        return str(self.chapter_id)

    def get_content(self):
        name = self.get_manga_name(False)
        return self.http_get('{}/manga/{}.html?waring=1'.format(self.domain, name))

    def get_manga_name(self, normalize=True) -> str:
        if not self._local_storage:
            name = self.re_name(self.get_url())
            if name:
                self._local_storage = name.group(1)
            else:
                url = self.html_fromstring(self.get_url(), '.subgiude > li + li > a', 0).get('href')
                self._local_storage = self.re_name(url).group(1)
        return self.normalize_name(self._local_storage, normalize)

    def get_chapters(self):
        result = self._elements('.chapterbox li a.chapter_list_a')
        items = []
        for i in result:
            u = self.re.search(r'(/chapter/.*/\d+)\.html', i.get('href'))
            items.append('{}{}-10-1.html'.format(self.domain, u.group(1)))
        return items

    def get_files_on_page(self, content):
        parser = self.document_fromstring(content)
        return self._images_helper(parser, 'img.manga_pic')

    def get_files(self):
        content = self._get_page_content(self.chapter)
        parser = self.document_fromstring(content)
        pages = self._first_select_options(parser, '.changepage #page')
        images = self.get_files_on_page(content)
        for i in pages:
            url = self.http().normalize_uri(i.get('value'))
            content = self._get_page_content(url)
            images += self.get_files_on_page(content)
        return images

    def prepare_cookies(self):
        self._base_cookies(self.get_url())

    def book_meta(self) -> dict:
        # todo meta
        pass


main = NineMangaCom



from logging import info
from manga_py.provider import Provider
from .helpers.std import Std
from ..crypt.bato_to_crypt import BatoToCrypt


class BatoTo(Provider, Std):

    def get_chapter_index(self) -> str:
        return '{}-{}'.format(
            self.chapter_id,
            self.chapter[1],
        )

    def get_content(self):
        url = self.get_url()
        if ~url.find('/chapter/'):
            url = self.html_fromstring(url, '.nav-path .nav-title > a', 0).get('href')
        return self.http_get(url)

    def get_manga_name(self) -> str:
        selector = '.nav-path .nav-title > a,.title-set .item-title > a'
        content = self.http_get(self.get_url())
        return self.text_content(content, selector, 0)

    def get_chapters(self):
        items = self._elements('.main > .item > a')
        n = self.http().normalize_uri
        result = []
        for i in items:
            title = i.cssselect('b')[0].text_content().strip(' \n\t\r')
            if ~title.find('DELETED'):  # SKIP DELETED
                continue
            result.append((
                n(i.get('href')),
                title,
            ))
        return result

    @staticmethod
    def _sort_files(data):
        keys = sorted(data, key=lambda _: int(_))
        return [data[i] for i in keys]

    def get_files(self):
        content = self.http_get(self.chapter[0])
        data = self.re.search(r'\simages\s*=\s*(\{.+});', content)
        try:
            return self._sort_files(self.json.loads(data.group(1)))
        except AttributeError:  # new format
            server = BatoToCrypt.decrypt_server(content)
            server = self.json.loads(server)

            images = self.re.search(r'\simages\s*=\s*(\[.+\]);', content).group(1)
            images = self.json.loads(images)

            n = self.http().normalize_uri
            info([n(f'{server}{img}') for img in images])
            return [n(f'{server}{img}') for img in images]
        except ValueError as e:
            self.log(f'Bato.to get_files error: {e}')
            return []

    def get_cover(self) -> str:
        return self._cover_from_content('.attr-cover img')

    def book_meta(self) -> dict:
        # todo meta
        pass

    def chapter_for_json(self):
        return self.chapter[0]


main = BatoTo



from .gomanga_co import GoMangaCo
from re import MULTILINE
from manga_py.crypt.base_lib import BaseLib


class JaiMiniBox(GoMangaCo):

    def get_files(self):
        self._go_chapter_content = self.http_get(self.chapter)

        selector = self.re.search(r'page_width\s=\sparseInt\((\w+)\[', self._go_chapter_content).group(1)
        selector_content = r'^\s*(.+var\s{}\s*=.+)$'.format(selector)

        atob = """
        const btoa = (str) => Buffer.from(str).toString('base64');
        const atob = (b64Encoded) => Buffer.from(b64Encoded, 'base64').toString('utf8');
        """

        content = self.re.search(selector_content, self._go_chapter_content, MULTILINE).group(1)

        items = BaseLib.exec_js('{}{}'.format(atob, content), 'JSON.stringify({})'.format(selector))

        items = self.json.loads(items)

        return [i.get('url') for i in items]


main = JaiMiniBox



from manga_py.provider import Provider
from .helpers.std import Std


class LeviatanScansCom(Provider, Std):

    def get_chapter_index(self) -> str:
        return self.re.search(r'.+/(\d+/\d+)', self.chapter).group(1).replace('/', '-')

    def get_content(self):
        return self.http_get(self.get_url())

    def get_manga_name(self) -> str:
        return self.text_content(self.content, '.text-highlight')

    def get_chapters(self):
        return self._elements('.list a.item-author')

    def get_files(self):
        content = self.http_get(self.chapter)
        images = self.re.search(r'chapterPages\s*=\s*(\[.+?\])', content).group(1)
        return self.json.loads(images)

    def get_cover(self) -> str:
        image = self._elements('.media-comic-card .media-content')[0]
        return self.parse_background(image)


main = LeviatanScansCom



from manga_py.provider import Provider
from .helpers.std import Std


class MangaLifeUs(Provider, Std):
    img_selector = '.image-container .CurImage'

    def get_chapter_index(self) -> str:
        selector = r'-chapter-(\d+)-index-(\d+)'
        chapter = self.re.search(selector, self.chapter)

        if chapter is None:  # http://mangalife.us/manga/Ubau-Mono-Ubawareru-Mono  #51
            selector = r'-chapter-(\d+(?:\.\d+)?)'
            chapter = self.re.search(selector, self.chapter).group(1).split('.')
            return '-'.join(chapter)

        chapter = chapter.groups()

        return '{}-{}'.format(
            chapter[0],
            1 if chapter[1] is None else chapter[1],
        )

    def get_content(self):
        return self._get_content('{}/manga/{}')

    def get_manga_name(self) -> str:
        uri = self.get_url()
        test = self.re.search(r'\.\w{2,7}/read-online/', uri)
        if test:
            uri = self.html_fromstring(uri, 'a.list-link', 0).get('href')
        return self.re.search(r'(?:\.\w{2,7})?/manga/([^/]+)', uri).group(1)

    def _chapters_js(self):
        raw_chapters = self.re.search(r'vm.Chapters\s*=\s*(\[\{.+\}\])', self.content).group(1)
        chapters = self.json.loads(raw_chapters)

        return ['{}/read-online/{}-chapter-{}-index-{}.html'.format(
            self.domain,
            self.manga_name,
            self.__ch(ch['Chapter']),
            ch['Chapter'][0],  # example mangasee123.com/manga/Tower-Of-God
        ) for ch in chapters]

    def _chapters_html(self):
        return self._elements('.list a.list-group-item')

    def get_chapters(self):
        try:
            return self._chapters_js()
        except:
            return self._chapters_html()

    def _files_js(self, content):
        domain = self.re.search(r"""vm.CurPathName\s*=\s*["'](.+)['"]""", content).group(1)
        raw_chapter = self.json.loads(self.re.search(r"""vm.CurChapter\s*=\s*(\{.+\})""", content).group(1))

        chapter = raw_chapter['Chapter']
        directory = raw_chapter['Directory']
        pages = int(raw_chapter['Page'])

        if len(directory) > 0:
            directory += '/'

        return ['https://{}/manga/{}/{}{}-{}.png'.format(
            domain,
            self.manga_name,
            directory,
            self.__one_ch(chapter),
            '{:0>3}'.format(i)
        ) for i in range(1, pages + 1)]

    def _files_html(self, content):
        parser = self.document_fromstring(content)
        return self._images_helper(parser, '.image-container-manga > .fullchapimage > img')

    def get_files(self):
        _chapter = self.chapter
        try:
            _chapter = self.re.search(r'(.+)-page-\d+\.html', _chapter).group(1)
        except AttributeError:
            pass

        content = self.http_get('%s.html' % _chapter)

        try:
            return self._files_js(content)
        except:
            return self._files_html(content)

    @staticmethod
    def __one_ch(ch):
        chapter = ch[1:-1]
        if ch[-1] == '0':
            return chapter
        return '%s.%s' % (chapter, ch[-1])

    @staticmethod
    def __ch(ch):
        n = ch[1:-1].lstrip('0')
        if ch[-1] != '0':
            return '%s.%s' % (n, ch[-1])
        return '{:0>1}'.format(n)

    def prepare_cookies(self):
        self.http().cookies['FullPage'] = 'yes'

    def get_cover(self) -> str:
        return self._cover_from_content('.leftImage img')

    def book_meta(self) -> dict:
        # todo meta
        pass


main = MangaLifeUs



from manga_py.provider import Provider
from .helpers.std import Std


class ReadPtscansCom(Provider, Std):
    def get_chapter_index(self) -> str:
        chapter = self.chapter['title']
        return self.re.search(r'.+?(\d+(?:\.\d+)?)\s*$', chapter)\
            .group(1).replace('.', '-')

    def get_content(self):
        return self._get_content('{}/series/{}')

    def get_manga_name(self) -> str:
        return self._get_name('/series/([^/]+)')

    def get_chapters(self):
        elements = self._elements('.mdc-list-item .mdc-list-item__text > a')

        if len(elements) == 0:
            return []

        content = self.http_get(self.http().normalize_uri(elements[0].get('href')))
        re = self.re.search(r'"(/api/chapters.json.+?)"', content).group(1)
        json = self.http_get(self.http().normalize_uri(re))

        return self.json.loads(json)['results']

    def get_files(self):
        content = self.http_get(self.http().normalize_uri(self.chapter['manifest']))
        json = self.json.loads(content)

        return [i['href'] for i in json['readingOrder']]

    def get_cover(self) -> str:
        return self._cover_from_content('.mdc-layout-grid__cell > img')

    def chapter_for_json(self) -> str:
        return self.chapter['title']


main = ReadPtscansCom



from .abstract import Abstract
from .archive import Archive
from .base import Base
from .callbacks import Callbacks
from .cf_protect import cf_scrape
from .chapter_helper import ChapterHelper
from .static import Static
from .arc_name import ArchiveName



from manga_py.provider import Provider
from .helpers.std import Std
from sys import stderr


class HentaiPornsNet(Provider, Std):

    def get_archive_name(self) -> str:
        return 'archive'

    def get_chapter_index(self) -> str:
        return '0'

    def get_content(self):
        url = self.get_url()
        if ~url.find('/tag/'):
            self.log('Please, use target url', file=stderr)
            exit(1)
        return self.http_get(self.get_url())

    def get_manga_name(self) -> str:
        return self._get_name(r'//[^/]+/([^/]+)')

    def get_chapters(self):
        return [b'']

    def get_files(self):
        parser = self.document_fromstring(self.content)
        return self._images_helper(parser, '.gallery-item a', 'href')

    def get_cover(self) -> str:
        return self._cover_from_content('.post-thumbnail img')


main = HentaiPornsNet
